

CALL SYSPROC.SET_ROUTINE_OPTS('ISOLATION UR') @

CREATE PROCEDURE CHECKOUT_AND_GET_ALERT (
    IN I_USER_NAME VARCHAR(60),
    IN I_ALERT_ID BIGINT,
    OUT O_SQLCODE INTEGER,
    OUT O_SQLSTATE CHAR(5)  )
SPECIFIC CHECKOUT_AND_GET_A
DYNAMIC RESULT SETS 1
LANGUAGE SQL
-----------------------------------------------------------------------------
-- File:  CHECKOUT_AND_GET_ALERT
--
--
-- Description:
--
--
-- Called By:
--    force_checkout
--
-- Calls:
--
--
-- Notes:
--       Overloaded there are two versions of checkout_and_get_alert.
--       This version should only be called by the alert service engine itself.
--       It should not be called by the front end - no lstupdt_nbr parameter
--
--
-- Usage:
--		Called from the web UI
--
-- History:    10/17/2005:               taken from FCMINT and stored in PRISM
--             2005/12/12                added o_sqlcode and o_sqlstate parameters
--             2006/06/05                added updates to pause_timestamp & 
--                                           elapsed_pause_time
--             2006/06/06                added logical_del_ind checks
--             2009/08/14    leushe      Added the security check to ensure the user_name has access to update the alert
--             2010/04/26    leushe      Added logic to ensure alert suspension can occur, prior to checking out the alert
--			   2010/05/24    leushe      Added SUSPEND Alert Action insert
--			   2010/10/22    leushe      Changed references to GRX_, GRA_ and GRC_ to XQO_,AQO_, and blank respectively
--			   2011/03/30    leushe      Commented out the insert into FCM_ALERT_ACTION for SUSPEND actions
--             2011/04/01    leushe      Changed the FCM_ALERT_SUSPEND to fix race condition and addition of ALERT_CHECKOUT_CNT column
------------------------------------------------------------------------
P1: BEGIN

    DECLARE L_ROWCOUNT INTEGER DEFAULT -1;
    DECLARE APPSQLEXC_UPDATESINGLEROW    CONDITION FOR SQLSTATE '99002';
    DECLARE SQLSTATE CHAR(5) DEFAULT '00000';
    DECLARE SQLCODE INTEGER DEFAULT 0;
    DECLARE L_SUSPEND CHAR(5) DEFAULT 'false';
    DECLARE l_cust_num char(34);    
--
--Check to see if suspended alerts is on
--
SELECT coalesce(SUBSTR(LCASE(PROPERTY_VALUE),1,5),'false') INTO L_SUSPEND FROM FCM_PROPERTY WHERE PROPERTY_NAME = 
    	'analyst.alert.suspend.enabled';
    IF (L_SUSPEND is null) THEN
    	set L_SUSPEND = 'false';
    END IF;

    
    IF (L_SUSPEND = 'true') THEN
        INSERT INTO FCM_ALERT_SUSPEND (XQO_CUST_NUM,ALERT_ID,SUSPEND_USER,SUSPEND_TIMESTAMP)
		SELECT XQO_CUST_NUM,ALERT_ID,I_USER_NAME,CURRENT TIMESTAMP - CURRENT TIMEZONE 
			FROM FCM_ALERT A
			WHERE NOT EXISTS (SELECT 1 FROM FCM_ALERT_SUSPEND SUS
						WHERE A.XQO_CUST_NUM = SUS.XQO_CUST_NUM 
/* commented out for race condition*/
--						      AND SUS.SUSPEND_USER <> I_USER_NAME 
--							  AND SUS.ALERT_ID <> I_ALERT_ID
							  )
						AND A.ALERT_ID = I_ALERT_ID 
						AND A.ALERT_STATUS='ACTIVE';
		GET DIAGNOSTICS L_ROWCOUNT = ROW_COUNT;
		If(L_ROWCOUNT = 0) then
			select xqo_cust_num into L_CUST_NUM from fcm_alert where alert_id = i_alert_id;
			update fcm_alert_suspend
			  set ALERT_CHECKOUT_CNT = ALERT_CHECKOUT_CNT+1
			where xqo_cust_num = l_cust_num and suspend_user = i_user_name;
			
			GET DIAGNOSTICS L_ROWCOUNT = ROW_COUNT;
		end if;		
--		IF (L_ROWCOUNT > 0) THEN
--		  -- suspend the alert
--          INSERT INTO FCM_ALERT_ACTION (ALERT_ID ,
--			ALERT_VERSION_NBR ,
--			ACTION_TIMESTAMP ,
--			XQO_CUST_NUM ,
--			ALERT_VALUE ,
--			ALERT_TYPE ,
--			MULTI_ORG_ID, 
--			ALERT_STATUS ,
--			GMT_OFFSET ,
--			ALERT_SERVICE_CNT ,
--			ALERT_ORIGIN ,
--			CREATE_USER ,
--			CREATE_TIMESTAMP ) 
--			SELECT a.ALERT_ID ,
--				a.ALERT_VERSION_NBR ,
--				CURRENT TIMESTAMP - CURRENT TIMEZONE ,
--				a.XQO_CUST_NUM ,
--				a.ALERT_VALUE ,
--				a.ALERT_TYPE ,
--				a.MULTI_ORG_ID, 
--				'SUSPEND' ,
--				a.GMT_OFFSET ,
--				a.ALERT_SERVICE_CNT ,
--				a.ALERT_ORIGIN ,
--				a.CREATE_USER ,
--				a.CREATE_TIMESTAMP  
--			FROM FCM_ALERT a
--			WHERE a.ALERT_ID = I_ALERT_ID;
--		END IF;
    END IF;
						
    
   	 IF(L_SUSPEND='false' OR L_ROWCOUNT = 1) THEN

    UPDATE FCM_ALERT
      SET LSTUPDT_USER = I_USER_NAME,
          ALERT_STATUS = 'CHECKOUT',
          ALERT_SERVICE_CNT = ALERT_SERVICE_CNT + 1,
          CHECKOUT_USER = I_USER_NAME,
          USER_SERVICE_TIMESTAMP = CURRENT TIMESTAMP - CURRENT TIMEZONE,
          LSTUPDT_TIMESTAMP = CURRENT TIMESTAMP - CURRENT TIMEZONE,
          LSTUPDT_NBR = LSTUPDT_NBR + 1,
          CHECKOUT_TIMESTAMP = CURRENT TIMESTAMP - CURRENT TIMEZONE,
          PAUSE_TIMESTAMP = NULL,
          PAUSE_ELAPSED_TIME = 0
      WHERE ALERT_ID = I_ALERT_ID
      AND ALERT_STATUS='ACTIVE'
      AND LOGICAL_DEL_IND = '0'
      AND CHECK_MO_AT_ACCESS(I_USER_NAME,MULTI_ORG_ID, ALERT_TYPE) = 1 ;

   GET DIAGNOSTICS L_ROWCOUNT = ROW_COUNT;

   END IF;
   IF ( L_ROWCOUNT = 1 ) THEN

     COMMIT;
     CALL GET_ALERT(I_ALERT_ID, I_USER_NAME);
     SET O_SQLCODE = SQLCODE;
     SET O_SQLSTATE = SQLSTATE;

   ELSE
     -- eventually will need to signal an exception; now just return an empty set
     --SIGNAL appSqlExc_UpdateSingleRow SET MESSAGE_TEXT = 'FAILURE: Alert could not be checked out.';
     ROLLBACK;
     CALL GET_ALERT(-1, I_USER_NAME);
     SET O_SQLCODE = -1;
     SET O_SQLSTATE = 'ERROR';
   END IF;

END P1 @



CREATE PROCEDURE CHECKOUT_AND_GET_ALERT (
    IN I_USER_NAME VARCHAR(60),
    IN I_ALERT_ID BIGINT,
    IN I_LSTUPDT_NBR INTEGER,
    OUT O_SQLCODE INTEGER,
    OUT O_SQLSTATE CHAR(5) )
SPECIFIC CHECKOUT_AND_GET_2
DYNAMIC RESULT SETS 1
LANGUAGE SQL
-----------------------------------------------------------------------------
-- File:  CHECKOUT_AND_GET_ALERT.SQL
--
--
-- Description:
--
--
-- Called By:
--    Web UI
--
-- Calls:
--    GET_ALERT
--
-- Notes:
--       Overloaded there are two versions of checkout_and_get_alert.
--       This version is accessible to the front end.   It should not be called
--       by the alert service engine.

--
--
-- Usage:
--
--
-- History:    10/17/2005:  taken from FCMINT and stored in PRISM
--             2005/12/12   added i_lstupdt_nbr, o_sqlcode, & o_sqlstate parms
--             2006/06/05   added updates to pause_timestamp & 
--                             elapsed_pause_time
--             2010/04/26    leushe      Added logic to ensure alert suspension can occur, prior to checking out the alert
--			   2010/05/24    leushe      Added SUSPEND Alert Action insert
--			   2011/03/30    leushe      Commented out the insert into FCM_ALERT_ACTION for SUSPEND actions
--             2011/04/01    leushe      Changed the FCM_ALERT_SUSPEND to fix race condition and addition of ALERT_CHECKOUT_CNT column
--
--
------------------------------------------------------------------------
P1: BEGIN

    DECLARE L_ROWCOUNT INTEGER DEFAULT -1;
    DECLARE APPSQLEXC_UPDATESINGLEROW    CONDITION FOR SQLSTATE '99002';
    DECLARE SQLSTATE CHAR(5) DEFAULT '00000';
    DECLARE SQLCODE INTEGER DEFAULT 0;
    DECLARE L_SUSPEND CHAR(5) DEFAULT 'false';
    DECLARE l_cust_num char(34);
--
--Check to see if suspended alerts is on
--
SELECT  coalesce(SUBSTR(LCASE(PROPERTY_VALUE),1,5),'false')  INTO L_SUSPEND FROM FCM_PROPERTY WHERE PROPERTY_NAME = 
    	'analyst.alert.suspend.enabled';
    IF (L_SUSPEND is null) THEN
    	set L_SUSPEND = 'false';
    END IF;
    
    IF (L_SUSPEND = 'true') THEN
        INSERT INTO FCM_ALERT_SUSPEND (XQO_CUST_NUM,ALERT_ID,SUSPEND_USER,SUSPEND_TIMESTAMP)
		SELECT XQO_CUST_NUM,ALERT_ID,I_USER_NAME,CURRENT TIMESTAMP - CURRENT TIMEZONE 
			FROM FCM_ALERT A
			WHERE NOT EXISTS (SELECT 1 FROM FCM_ALERT_SUSPEND SUS
						WHERE A.XQO_CUST_NUM = SUS.XQO_CUST_NUM 
/* commented out for race condition*/
--						      AND SUS.SUSPEND_USER <> I_USER_NAME 
--							  AND SUS.ALERT_ID <> I_ALERT_ID
							  )
						AND A.ALERT_ID = I_ALERT_ID 
						AND A.ALERT_STATUS='ACTIVE';
		GET DIAGNOSTICS L_ROWCOUNT = ROW_COUNT;
		If(L_ROWCOUNT = 0) then
		    select xqo_cust_num into L_CUST_NUM from fcm_alert where alert_id = i_alert_id;
			update fcm_alert_suspend
			  set ALERT_CHECKOUT_CNT = ALERT_CHECKOUT_CNT+1
			where xqo_cust_num = L_CUST_NUM and suspend_user = i_user_name;
			
			GET DIAGNOSTICS L_ROWCOUNT = ROW_COUNT;
		end if;
--		IF (L_ROWCOUNT > 0) THEN
--				 -- suspend the alert
--          INSERT INTO FCM_ALERT_ACTION (ALERT_ID ,
--			ALERT_VERSION_NBR ,
--			ACTION_TIMESTAMP ,
--			XQO_CUST_NUM ,
--			ALERT_VALUE ,
--			ALERT_TYPE ,
--			MULTI_ORG_ID, 
--			ALERT_STATUS ,
--			GMT_OFFSET ,
--			ALERT_SERVICE_CNT ,
--			ALERT_ORIGIN ,
--			CREATE_USER ,
--			CREATE_TIMESTAMP ) 
--			SELECT a.ALERT_ID ,
--				a.ALERT_VERSION_NBR ,
--				CURRENT TIMESTAMP - CURRENT TIMEZONE ,
--				a.XQO_CUST_NUM ,
--				a.ALERT_VALUE ,
--				a.ALERT_TYPE ,
--				a.MULTI_ORG_ID, 
--				'SUSPEND' ,
--				a.GMT_OFFSET ,
--				a.ALERT_SERVICE_CNT ,
--				a.ALERT_ORIGIN ,
--				a.CREATE_USER ,
--				a.CREATE_TIMESTAMP  
--			FROM FCM_ALERT a
--			WHERE a.ALERT_ID = I_ALERT_ID;
--		END IF;		
    END IF;
						
    
   	 IF(L_SUSPEND='false' OR L_ROWCOUNT = 1) THEN
						
    UPDATE FCM_ALERT
      SET LSTUPDT_USER = I_USER_NAME,
          ALERT_STATUS = 'CHECKOUT',
          ALERT_SERVICE_CNT = ALERT_SERVICE_CNT + 1,
          CHECKOUT_USER = I_USER_NAME,
          USER_SERVICE_TIMESTAMP = CURRENT TIMESTAMP - CURRENT TIMEZONE,
          LSTUPDT_TIMESTAMP = CURRENT TIMESTAMP - CURRENT TIMEZONE,
          LSTUPDT_NBR = LSTUPDT_NBR + 1,
          CHECKOUT_TIMESTAMP = CURRENT TIMESTAMP - CURRENT TIMEZONE,
          PAUSE_TIMESTAMP = NULL,
          PAUSE_ELAPSED_TIME = 0
      WHERE ALERT_ID = I_ALERT_ID
      AND LSTUPDT_NBR = I_LSTUPDT_NBR
      AND ALERT_STATUS='ACTIVE'
      AND LOGICAL_DEL_IND = '0' ;

   GET DIAGNOSTICS L_ROWCOUNT = ROW_COUNT;
   END IF;
   IF ( L_ROWCOUNT = 1 ) THEN

     COMMIT;
     CALL GET_ALERT(I_ALERT_ID, I_USER_NAME);
     SET O_SQLCODE = SQLCODE;
     SET O_SQLSTATE = SQLSTATE;

   ELSE
     -- eventually will need to signal an exception; now just return an empty set
     --SIGNAL appSqlExc_UpdateSingleRow SET MESSAGE_TEXT = 'FAILURE: Alert could not be checked out.';
     ROLLBACK;
     CALL GET_ALERT(-1, I_USER_NAME);
     SET O_SQLCODE = -1;
     SET O_SQLSTATE = 'ERROR';
   END IF;

END P1 @

--GRANT EXECUTE ON SPECIFIC PROCEDURE CHECKOUT_AND_GET_2 TO ROLE APPROLE @
--
--GRANT EXECUTE ON SPECIFIC PROCEDURE CHECKOUT_AND_GET_A TO ROLE APPROLE @
