

CALL SYSPROC.SET_ROUTINE_OPTS ('isolation ur') @

CREATE PROCEDURE CR_MAN_ALERT (
    IN I_ALERT_VALUE CHARACTER(34),
    IN I_ACCT_NUM CHARACTER(34),
    IN I_CUST_NUM CHARACTER(34),
    IN I_QUEUE_ID BIGINT,
    IN I_USER_NAME CHARACTER(60),
    IN I_GROUP_CODE CHARACTER(2),
    IN I_MULTI_ORG_NAME VARCHAR(16),
    IN I_ALERT_TYPE CHAR(1),
    IN I_NEED_TO_COMMIT CHARACTER(1) )
  SPECIFIC CR_MAN_ALERT
  DYNAMIC RESULT SETS 1
  LANGUAGE SQL
-----------------------------------------------------------------------------
-- file:   cr_man_alert
--
--
-- description:  creates a manual alert, needed for inbound when alert doesn't exist.
--
--
-- called by: user interface inbound
--
--
-- calls:
--	   get_alert_row
--     get_alert
--     save_alert_action
--
-- notes:
--
--
-- usage:
--
--
-- history:    12/09/2005:  created by mike ames (x10866 michael.ames@sas.com )
--             2006/01/16:  jicimi   added fulfil_text_proof_code_[1,2,3] columns
--             2006/03/15:  jicimi   active status changed to checkout
--             2006/06/22:  jicimi   logic to handle preexisting alerts added
--             2006/06/28:  jicimi   added call to get_alert instead of using
--                                   the inline cursor (ui object consistency)
--             2006/08/23:  jicimi   action_type always alertinit
--                                   added predicate to alert table update statement
--                                   if alert was previously closed.
--             2007/02/12   jicimi   added strategy_id
--             2007/03/23            changed action_type value when a manual alert is
--                                   created.
--             2007/04/19   jicimi   changed multi_org_id & acct_gmt_lookup to use
--                                   a default if one isn't found.
--	           2007/06/19   sas      adapted for r2 (initial pass)
--             2007/07/17   leball   further adaptation for r2 data model
--             2007/09/21   jicimi   changed to sync up with save alert action proc
--             2007/09/25            multi_org_name changes;  updated signature and proc logic
--             2007/10/01            added acct_num & cust_num parameters
--             2007/10/23            changed 1 reference of fcm_auth to fcm_raur
--             2008/07/07   leushe   check to both credit and debit chip auth tables
--             2008/07/28   leushe   check for null or empty string i_alert_value or i_acct_num values passed in
--             2008/11/20   leushe   incorporated alert_level into the proc
--             2009/02/05   leushe   incorporated alert_version_nbr into the proc. this includes the call to save_alert_action.
--             2009/08/14   leushe   Added the security check to ensure the user_name has access to update the alert
--             2009/10/15   leushe   Changed the alert search query to only look for alerts based on alert_value and alert_type
--             2010/04/14   leushe   Updated the call signature to save_alert_action to not send pend_reason
--             2010/04/26   leushe   changed code, that is an alert is being re-initted, the grx_cust_num is not already suspended by another user
--      	   2010/05/24   leushe   Added SUSPEND Alert Action insert
--             2010/07/29   leushe   Change the alert_Version_nbr to increment by 1, if the alert was closed, and is being re-initted
--			   2010/10/22   leushe      Changed references to GRX_, GRA_ and GRC_ to XQO_,AQO_, and blank respectively
--			   2011/03/30    leushe      Commented out the insert into FCM_ALERT_ACTION for SUSPEND actions
--             2011/04/01    leushe      Changed the FCM_ALERT_SUSPEND to fix race condition and addition of ALERT_CHECKOUT_CNT column
--  
------------------------------------------------------------------------
CRALERT: BEGIN

-- step 0. declare local variables

DECLARE   L_MULTI_ORG_ID  INTEGER DEFAULT NULL;
DECLARE   L_GMT_OFFSET DECIMAL(3,1) DEFAULT 0.0;
DECLARE   L_ALERT_TYPE CHAR(1);
DECLARE   L_ALERT_ID BIGINT;
DECLARE   L_ALRT_VER_NBR INTEGER;
DECLARE   L_ALERT_VALUE  CHARACTER(34);
DECLARE   L_ACCT_NUM  CHARACTER(34);
DECLARE   L_CUST_NUM  CHARACTER(34);
DECLARE   L_QUEUE_ID BIGINT;
DECLARE   L_RESULT_ID  INTEGER;
DECLARE   L_ALERT_STATUS CHARACTER(10);
DECLARE   L_FRAUD_TYPE_CODE  CHARACTER(3);
DECLARE   L_PEND_REASON_CODE CHARACTER(20);
DECLARE   L_FRAUD_SCORE  INTEGER;
DECLARE   L_FRAUD_SCORE_REASON_CODE_1  INTEGER;
DECLARE   L_FRAUD_SCORE_REASON_CODE_2   INTEGER;
DECLARE   L_FRAUD_SCORE_REASON_CODE_3   INTEGER;
DECLARE   L_INIT_FRAUD_SCORE INTEGER;
DECLARE   L_INIT_FRAUD_SCORE_REASON_CODE_1  INTEGER;
DECLARE   L_INIT_FRAUD_SCORE_REASON_CODE_2  INTEGER;
DECLARE   L_INIT_FRAUD_SCORE_REASON_CODE_3  INTEGER;
DECLARE   L_HWM_FRAUD_SCORE  INTEGER;
DECLARE   L_HWM_FRAUD_SCORE_REASON_CODE_1   INTEGER;
DECLARE   L_HWM_FRAUD_SCORE_REASON_CODE_2   INTEGER;
DECLARE   L_HWM_FRAUD_SCORE_REASON_CODE_3   INTEGER;
DECLARE   L_ALERT_SERVICE_CNT  INTEGER;
DECLARE   L_ALERT_ORIGIN VARCHAR(20);
DECLARE   L_LST_TRANSACTION_AMT  DECIMAL(19,5);
DECLARE   L_LST_TRANSACTION_TIMESTAMP  TIMESTAMP;
DECLARE   L_USER_SERVICE_TIMESTAMP TIMESTAMP;
DECLARE   L_SYSTEM_SERVICE_TIMESTAMP TIMESTAMP;
DECLARE   L_NEXT_SERVICE_TIMESTAMP TIMESTAMP;
DECLARE   L_QUEUE_ENTRY_TIMESTAMP  TIMESTAMP;
DECLARE   L_FINAL_DISPOSITION_TIMESTAMP  TIMESTAMP;
DECLARE   L_CHECKOUT_USER  CHARACTER(60);
DECLARE   L_CHECKOUT_TIMESTAMP TIMESTAMP;
DECLARE   L_CREATE_USER  CHARACTER(60);
DECLARE   L_CREATE_TIMESTAMP TIMESTAMP;
DECLARE   L_LSTUPDT_USER CHARACTER(60);
DECLARE   L_LSTUPDT_TIMESTAMP  TIMESTAMP;
DECLARE   L_LSTUPDT_NBR  SMALLINT;
DECLARE   L_LOGICAL_DEL_IND  CHARACTER(1);
DECLARE   L_BASE_RULE_ID BIGINT;
DECLARE   L_RULE_VERSION_NBR INTEGER;
DECLARE   L_STRATEGY_ID BIGINT;
DECLARE   L_PAUSE_TIMESTAMP  TIMESTAMP;
DECLARE   L_PAUSE_ELAPSED_TIME INTEGER;
DECLARE   L_ACTION_TYPE CHAR(20);
DECLARE   L_CURRENT_UTC_TIMESTAMP TIMESTAMP;
DECLARE   P_ALERT_ID BIGINT;
DECLARE   L_SUSPEND CHAR(5) DEFAULT 'false';
DECLARE   L_ERROR INT DEFAULT -1;
DECLARE   L_ROW_COUNT INT DEFAULT -1;

DECLARE C_ALERTEXISTCURSOR CURSOR FOR
SELECT COALESCE(ALERT_ID,-1), ALERT_STATUS,XQO_CUST_NUM
FROM FCM_ALERT
WHERE ALERT_VALUE = I_ALERT_VALUE
AND ALERT_TYPE = L_ALERT_TYPE;



--
--Check to see if suspended alerts is on
--
SELECT coalesce(SUBSTR(LCASE(PROPERTY_VALUE),1,5),'false')   INTO L_SUSPEND FROM FCM_PROPERTY WHERE PROPERTY_NAME = 
    	'analyst.alert.suspend.enabled';
    
    IF (L_SUSPEND is null) THEN
    	set L_SUSPEND = 'false';
    END IF;
    
--
--Check for a valid alert_type
--
SELECT ALERT_TYPE
INTO L_ALERT_TYPE
FROM FCM_ALERT_TYPE
WHERE ALERT_TYPE = I_ALERT_TYPE;

--
--If alert_type is not valid, return 0
--
IF ((L_ALERT_TYPE IS NULL) OR (LENGTH(TRIM(L_ALERT_TYPE)) = 0)) THEN
	RETURN 0;
END IF;

--
--Check for a valid Multi_org. If not, use root multi_org_id
--

SELECT MULTI_ORG_ID
		INTO L_MULTI_ORG_ID
		FROM FCM_MULTI_ORG
		WHERE MULTI_ORG_NAME = I_MULTI_ORG_NAME;

IF ( L_MULTI_ORG_ID IS NULL ) THEN
		SELECT MULTI_ORG_ID
		INTO L_MULTI_ORG_ID
		FROM FCM_MULTI_ORG
		WHERE PARENT_MULTI_ORG_ID IS NULL;
END IF;

--
--Check Security Access to the Alert Type and Multi-org. If no access, send empty result set
--
IF (CHECK_MO_AT_ACCESS(I_USER_NAME,L_MULTI_ORG_ID,L_ALERT_TYPE)=1)
THEN
			--
			-- check for null input variables for i_alert_value ,  i_alert_type and i_cust_num
			--
			IF ((I_ALERT_VALUE IS NULL) OR (LENGTH(TRIM(I_ALERT_VALUE)) = 0)) THEN
			RETURN 0;
			END IF;
			
			IF ((I_CUST_NUM IS NULL) OR (LENGTH(TRIM(I_CUST_NUM)) = 0)) THEN
			RETURN 0;
			END IF;
			
			IF ((I_ALERT_TYPE IS NULL) OR (LENGTH(TRIM(I_ALERT_TYPE)) = 0)) THEN
			RETURN 0;
			END IF;
			

			
			-- 2007/09/25 changes
			-- step 1a. get  gmt offset
			-- Default to 0, if the customer is not found
			SELECT
				COALESCE(GMT_OFFSET,0.0) INTO L_GMT_OFFSET
			FROM FCM_CUST
			WHERE XQO_CUST_NUM = I_CUST_NUM;
			

			
			--Set the current timestamp once
			SET L_CURRENT_UTC_TIMESTAMP = CURRENT TIMESTAMP - CURRENT TIMEZONE;
			
			OPEN C_ALERTEXISTCURSOR;
			FETCH FROM C_ALERTEXISTCURSOR INTO L_ALERT_ID, L_ALERT_STATUS,L_CUST_NUM;
			CLOSE C_ALERTEXISTCURSOR;
			
			IF ( L_ALERT_ID != -1 ) THEN
					IF ( L_ALERT_STATUS = 'ACTIVE' ) THEN
							--
							-- in this case should a manual alert supercede an automated one?
							-- we probably have much more info on the alert that was created from the ose
							--    ( a queue, fraud score, fraud reason, ect...)
							-- current alert should be left unchanged at this point
							--
							SET P_ALERT_ID = L_ALERT_ID;
			
					ELSEIF ( L_ALERT_STATUS = 'CHECKOUT' ) THEN
								--
								-- write alert action row
								--
								CALL GET_ALERT_ROW
									( L_ALERT_ID
										,L_ALRT_VER_NBR
										,L_ALERT_VALUE
										,L_ACCT_NUM
										,L_CUST_NUM
										,L_QUEUE_ID
										,L_MULTI_ORG_ID
										,L_ALERT_TYPE
										,L_RESULT_ID
										,L_ALERT_STATUS
										,L_GMT_OFFSET
										,L_FRAUD_TYPE_CODE
										,L_FRAUD_SCORE
										,L_FRAUD_SCORE_REASON_CODE_1
										,L_FRAUD_SCORE_REASON_CODE_2
										,L_FRAUD_SCORE_REASON_CODE_3
										,L_INIT_FRAUD_SCORE
										,L_INIT_FRAUD_SCORE_REASON_CODE_1
										,L_INIT_FRAUD_SCORE_REASON_CODE_2
										,L_INIT_FRAUD_SCORE_REASON_CODE_3
										,L_HWM_FRAUD_SCORE
										,L_HWM_FRAUD_SCORE_REASON_CODE_1
										,L_HWM_FRAUD_SCORE_REASON_CODE_2
										,L_HWM_FRAUD_SCORE_REASON_CODE_3
										,L_ALERT_SERVICE_CNT
										,L_ALERT_ORIGIN
										,L_LST_TRANSACTION_AMT
										,L_LST_TRANSACTION_TIMESTAMP
										,L_USER_SERVICE_TIMESTAMP
										,L_SYSTEM_SERVICE_TIMESTAMP
										,L_NEXT_SERVICE_TIMESTAMP
										,L_QUEUE_ENTRY_TIMESTAMP
										,L_FINAL_DISPOSITION_TIMESTAMP
										,L_CHECKOUT_USER
										,L_CHECKOUT_TIMESTAMP
										,I_USER_NAME
										,L_CREATE_TIMESTAMP
										,I_USER_NAME
										,L_LSTUPDT_TIMESTAMP
										,L_LSTUPDT_NBR
										,L_BASE_RULE_ID
										,L_RULE_VERSION_NBR
										,L_PAUSE_TIMESTAMP
										,L_PAUSE_ELAPSED_TIME
										);
					
					CALL SAVE_ALERT_ACTION
										( L_ALERT_ID
										,L_ALRT_VER_NBR
										,L_ALERT_VALUE
										,L_ACCT_NUM
										,L_CUST_NUM
										,L_QUEUE_ID
										,L_MULTI_ORG_ID
										,L_ALERT_TYPE
										,L_RESULT_ID
										,L_ALERT_STATUS
										,L_GMT_OFFSET
										,L_FRAUD_TYPE_CODE
										,L_FRAUD_SCORE
										,L_FRAUD_SCORE_REASON_CODE_1
										,L_FRAUD_SCORE_REASON_CODE_2
										,L_FRAUD_SCORE_REASON_CODE_3
										,L_INIT_FRAUD_SCORE
										,L_INIT_FRAUD_SCORE_REASON_CODE_1
										,L_INIT_FRAUD_SCORE_REASON_CODE_2
										,L_INIT_FRAUD_SCORE_REASON_CODE_3
										,L_HWM_FRAUD_SCORE
										,L_HWM_FRAUD_SCORE_REASON_CODE_1
										,L_HWM_FRAUD_SCORE_REASON_CODE_2
										,L_HWM_FRAUD_SCORE_REASON_CODE_3
										,L_ALERT_SERVICE_CNT
										,L_ALERT_ORIGIN
										,L_LST_TRANSACTION_AMT
										,L_LST_TRANSACTION_TIMESTAMP
										,L_USER_SERVICE_TIMESTAMP
										,L_SYSTEM_SERVICE_TIMESTAMP
										,L_NEXT_SERVICE_TIMESTAMP
										,L_QUEUE_ENTRY_TIMESTAMP
										,L_FINAL_DISPOSITION_TIMESTAMP
										,L_CHECKOUT_USER
										,L_CHECKOUT_TIMESTAMP
										,I_USER_NAME
										,L_CREATE_TIMESTAMP
										,I_USER_NAME
										,L_LSTUPDT_TIMESTAMP
										,L_LSTUPDT_NBR
								--		,NULL --,l_pend_reason_code
										,NULL --,l_reroute_queue_id
										,'INB' -- ,l_service_method_code
										,'ALERTINIT' --l_action_type
										,'N' -- l_need_to_commit
										,L_BASE_RULE_ID
										,L_RULE_VERSION_NBR
										,L_PAUSE_TIMESTAMP
										,L_PAUSE_ELAPSED_TIME
										,NULL -- ,l_strategy_id
						);
					
						SET P_ALERT_ID = L_ALERT_ID;
					
					ELSEIF ( L_ALERT_STATUS = 'CLOSED' ) THEN
								--
								-- update fcm_alert set status = 'ACTIVE'
								--
								IF (L_SUSPEND = 'true') THEN
			        				INSERT INTO FCM_ALERT_SUSPEND (XQO_CUST_NUM,ALERT_ID,SUSPEND_USER,SUSPEND_TIMESTAMP)
									SELECT L_CUST_NUM,ALERT_ID,I_USER_NAME,CURRENT TIMESTAMP - CURRENT TIMEZONE 
									FROM FCM_ALERT
									WHERE NOT EXISTS (SELECT 1 FROM FCM_ALERT_SUSPEND 
														WHERE XQO_CUST_NUM = I_CUST_NUM 
/* commented out for race condition*/														
--									      				AND SUSPEND_USER <> I_USER_NAME 
--										  				AND ALERT_ID <> L_ALERT_ID
										  				)
										AND ALERT_ID = L_ALERT_ID 
										AND ALERT_STATUS='CLOSED';
			 						GET DIAGNOSTICS L_ROW_COUNT = ROW_COUNT;


									If(l_row_count = 0) then
										update fcm_alert_suspend
										  set ALERT_CHECKOUT_CNT = ALERT_CHECKOUT_CNT+1
										where xqo_cust_num = i_cust_num and suspend_user = i_user_name;
			
										GET DIAGNOSTICS l_row_count = ROW_COUNT;
									end if;	
									IF(L_ROW_COUNT <> 1) THEN
										SET L_ERROR = 1;
										SET L_ALERT_ID = -1;
									END IF;
								END IF;		
								IF (L_ERROR <> 1) THEN
								UPDATE FCM_ALERT SET
										QUEUE_ID = I_QUEUE_ID
										,ALERT_VERSION_NBR = ALERT_VERSION_NBR + 1
										,MULTI_ORG_ID = L_MULTI_ORG_ID
										,XQO_CUST_NUM = I_CUST_NUM
										,AQO_ACCT_NUM = I_ACCT_NUM
										,ALERT_TYPE =  L_ALERT_TYPE
										,ALERT_STATUS = 'CHECKOUT'
										,GMT_OFFSET = L_GMT_OFFSET
										,FRAUD_SCORE = L_FRAUD_SCORE
										,FRAUD_SCORE_REASON_CODE_1 = L_FRAUD_SCORE_REASON_CODE_1
										,INIT_FRAUD_SCORE  = L_FRAUD_SCORE
										,INIT_FRAUD_SCORE_REASON_CODE_1  = L_FRAUD_SCORE_REASON_CODE_1
										,HWM_FRAUD_SCORE  = L_FRAUD_SCORE
										,HWM_FRAUD_SCORE_REASON_CODE_1  = L_FRAUD_SCORE_REASON_CODE_1
										,ALERT_SERVICE_CNT = 1
										,ALERT_ORIGIN = 'INBOUND'
										,INIT_TRANSACTION_AMT = L_LST_TRANSACTION_AMT
										,INIT_TRANSACTION_TIMESTAMP = L_CURRENT_UTC_TIMESTAMP
										,LST_TRANSACTION_AMT = L_LST_TRANSACTION_AMT
										,LST_TRANSACTION_TIMESTAMP   = L_CURRENT_UTC_TIMESTAMP
										,USER_SERVICE_TIMESTAMP   = L_CURRENT_UTC_TIMESTAMP
										,SYSTEM_SERVICE_TIMESTAMP   = L_CURRENT_UTC_TIMESTAMP
										,NEXT_SERVICE_TIMESTAMP   = L_CURRENT_UTC_TIMESTAMP
										,QUEUE_ENTRY_TIMESTAMP   = L_CURRENT_UTC_TIMESTAMP
										,CHECKOUT_USER = I_USER_NAME
										,CHECKOUT_TIMESTAMP   = L_CURRENT_UTC_TIMESTAMP
										,CREATE_USER = I_USER_NAME
										,CREATE_TIMESTAMP   = L_CURRENT_UTC_TIMESTAMP
										,LSTUPDT_USER = I_USER_NAME
										,LSTUPDT_TIMESTAMP   = L_CURRENT_UTC_TIMESTAMP
										,LSTUPDT_NBR = 0
										,LOGICAL_DEL_IND = '0'
									WHERE ALERT_ID = L_ALERT_ID;
					
								CALL GET_ALERT_ROW
										( L_ALERT_ID
										,L_ALRT_VER_NBR
										,L_ALERT_VALUE
										,L_ACCT_NUM
										,L_CUST_NUM
										,L_QUEUE_ID
										,L_MULTI_ORG_ID
										,L_ALERT_TYPE
										,L_RESULT_ID
										,L_ALERT_STATUS
										,L_GMT_OFFSET
										,L_FRAUD_TYPE_CODE
										,L_FRAUD_SCORE
										,L_FRAUD_SCORE_REASON_CODE_1
										,L_FRAUD_SCORE_REASON_CODE_2
										,L_FRAUD_SCORE_REASON_CODE_3
										,L_INIT_FRAUD_SCORE
										,L_INIT_FRAUD_SCORE_REASON_CODE_1
										,L_INIT_FRAUD_SCORE_REASON_CODE_2
										,L_INIT_FRAUD_SCORE_REASON_CODE_3
										,L_HWM_FRAUD_SCORE
										,L_HWM_FRAUD_SCORE_REASON_CODE_1
										,L_HWM_FRAUD_SCORE_REASON_CODE_2
										,L_HWM_FRAUD_SCORE_REASON_CODE_3
										,L_ALERT_SERVICE_CNT
										,L_ALERT_ORIGIN
										,L_LST_TRANSACTION_AMT
										,L_LST_TRANSACTION_TIMESTAMP
										,L_USER_SERVICE_TIMESTAMP
										,L_SYSTEM_SERVICE_TIMESTAMP
										,L_NEXT_SERVICE_TIMESTAMP
										,L_QUEUE_ENTRY_TIMESTAMP
										,L_FINAL_DISPOSITION_TIMESTAMP
										,L_CHECKOUT_USER
										,L_CHECKOUT_TIMESTAMP
										,I_USER_NAME
										,L_CREATE_TIMESTAMP
										,I_USER_NAME
										,L_LSTUPDT_TIMESTAMP
										,L_LSTUPDT_NBR
										,L_BASE_RULE_ID
										,L_RULE_VERSION_NBR
										,L_PAUSE_TIMESTAMP
										,L_PAUSE_ELAPSED_TIME
										);
					          INSERT INTO FCM_ALERT_ACTION (ALERT_ID ,
									ALERT_VERSION_NBR ,
									ACTION_TIMESTAMP ,
									XQO_CUST_NUM ,
									ALERT_VALUE ,
									ALERT_TYPE ,
									MULTI_ORG_ID, 
									ALERT_STATUS ,
									GMT_OFFSET ,
									ALERT_SERVICE_CNT ,
									ALERT_ORIGIN ,
									CREATE_USER ,
									CREATE_TIMESTAMP ) 
							VALUES( L_ALERT_ID ,
									L_ALRT_VER_NBR ,
									L_CURRENT_UTC_TIMESTAMP ,
									L_CUST_NUM ,
									l_ALERT_VALUE ,
									l_ALERT_TYPE ,
									L_MULTI_ORG_ID, 
									'SUSPEND' ,
									l_GMT_OFFSET ,
									L_ALERT_SERVICE_CNT ,
									l_ALERT_ORIGIN ,
									I_USER_NAME ,
									l_CREATE_TIMESTAMP  );
				
								CALL SAVE_ALERT_ACTION
										( L_ALERT_ID
											,L_ALRT_VER_NBR
											,L_ALERT_VALUE
											,L_ACCT_NUM
											,L_CUST_NUM
											,L_QUEUE_ID
											,L_MULTI_ORG_ID
											,L_ALERT_TYPE
											,L_RESULT_ID
											,L_ALERT_STATUS
											,L_GMT_OFFSET
											,L_FRAUD_TYPE_CODE
											,L_FRAUD_SCORE
											,L_FRAUD_SCORE_REASON_CODE_1
											,L_FRAUD_SCORE_REASON_CODE_2
											,L_FRAUD_SCORE_REASON_CODE_3
											,L_INIT_FRAUD_SCORE
											,L_INIT_FRAUD_SCORE_REASON_CODE_1
											,L_INIT_FRAUD_SCORE_REASON_CODE_2
											,L_INIT_FRAUD_SCORE_REASON_CODE_3
											,L_HWM_FRAUD_SCORE
											,L_HWM_FRAUD_SCORE_REASON_CODE_1
											,L_HWM_FRAUD_SCORE_REASON_CODE_2
											,L_HWM_FRAUD_SCORE_REASON_CODE_3
											,L_ALERT_SERVICE_CNT
											,L_ALERT_ORIGIN
											,L_LST_TRANSACTION_AMT
											,L_LST_TRANSACTION_TIMESTAMP
											,L_USER_SERVICE_TIMESTAMP
											,L_SYSTEM_SERVICE_TIMESTAMP
											,L_NEXT_SERVICE_TIMESTAMP
											,L_QUEUE_ENTRY_TIMESTAMP
											,L_FINAL_DISPOSITION_TIMESTAMP
											,L_CHECKOUT_USER
											,L_CHECKOUT_TIMESTAMP
											,I_USER_NAME
											,L_CREATE_TIMESTAMP
											,I_USER_NAME
											,L_LSTUPDT_TIMESTAMP
											,L_LSTUPDT_NBR
										--	,NULL --,l_pend_reason_code
											,NULL --,l_reroute_queue_id
											,'INB' -- ,l_service_method_code
											,'INITCLOSED' --l_action_type    (need to specify manual?)
											,'N' -- l_need_to_commit
											,L_BASE_RULE_ID
											,L_RULE_VERSION_NBR
											,L_PAUSE_TIMESTAMP
											,L_PAUSE_ELAPSED_TIME
											,NULL --,l_strategy_id
								);
								END IF;
						SET P_ALERT_ID = L_ALERT_ID;
					
					END IF;
			
			ELSE
			
			
					-- normal logic flow, ie an alert doesn't already exist for this card
					-- step 2. insert alert
					IF (L_SUSPEND = 'true') THEN
					    INSERT INTO FCM_ALERT_SUSPEND (XQO_CUST_NUM,SUSPEND_USER,SUSPEND_TIMESTAMP)
						SELECT I_CUST_NUM,I_USER_NAME,CURRENT TIMESTAMP - CURRENT TIMEZONE 
						FROM SYSIBM.SYSDUMMY1
						WHERE NOT EXISTS (SELECT 1 FROM FCM_ALERT_SUSPEND 
											WHERE XQO_CUST_NUM = I_CUST_NUM 
/* commented out for race condition*/											
--						      				AND SUSPEND_USER <> I_USER_NAME 
--							  				AND ALERT_ID <> P_ALERT_ID
										);
--							AND ALERT_ID = P_ALERT_ID 
--							AND ALERT_STATUS='CHECKOUT';
							GET DIAGNOSTICS l_row_count = ROW_COUNT;
 							If(l_row_count = 0) then
										update fcm_alert_suspend
										  set ALERT_CHECKOUT_CNT = ALERT_CHECKOUT_CNT+1
										where xqo_cust_num = i_cust_num and suspend_user = i_user_name;
			
										GET DIAGNOSTICS l_row_count = ROW_COUNT;
							end if;		
						IF (l_row_count <> 1)THEN
						     SET L_ERROR = 1;
						     SET P_ALERT_ID = -1;
						 END IF;
					END IF;
					IF (L_ERROR <> 1) THEN
					INSERT INTO FCM_ALERT(
						ALERT_VALUE,
						AQO_ACCT_NUM,
						XQO_CUST_NUM,
						QUEUE_ID,
						MULTI_ORG_ID,
						ALERT_TYPE,
						ALERT_STATUS,
						GMT_OFFSET,
						--			FRAUD_SCORE,
						--			FRAUD_SCORE_REASON_CODE_1,
						--			INIT_FRAUD_SCORE,
						--			INIT_FRAUD_SCORE_REASON_CODE_1,
						--			HWM_FRAUD_SCORE,
						--			HWM_FRAUD_SCORE_REASON_CODE_1,
						ALERT_SERVICE_CNT,
						ALERT_ORIGIN,
						--			INIT_TRANSACTION_AMT,
						--			INIT_TRANSACTION_TIMESTAMP,
						--			LST_TRANSACTION_AMT,
						--			LST_TRANSACTION_TIMESTAMP,
						USER_SERVICE_TIMESTAMP,
						SYSTEM_SERVICE_TIMESTAMP,
						NEXT_SERVICE_TIMESTAMP,
						QUEUE_ENTRY_TIMESTAMP,
						CHECKOUT_USER,
						CHECKOUT_TIMESTAMP,
						CREATE_USER,
						CREATE_TIMESTAMP,
						LSTUPDT_USER,
						LSTUPDT_TIMESTAMP,
						LSTUPDT_NBR,
						LOGICAL_DEL_IND
						--,base_rule_id
						--,rule_version_nbr
									)
					VALUES(
						I_ALERT_VALUE,
						I_ACCT_NUM,
						I_CUST_NUM,
						I_QUEUE_ID,
						L_MULTI_ORG_ID,
						L_ALERT_TYPE,
						'CHECKOUT',
						L_GMT_OFFSET,
						--			L_FRAUD_SCORE,
						--			L_FRAUD_SCORE_REASON_CODE_1,
						--			L_FRAUD_SCORE,
						--			L_FRAUD_SCORE_REASON_CODE_1,
						--			L_FRAUD_SCORE,
						--			L_FRAUD_SCORE_REASON_CODE_1,
						1,
						'INBOUND',
						--			L_LST_TRANSACTION_AMT,
						--			L_CURRENT_UTC_TIMESTAMP,
						--			L_LST_TRANSACTION_AMT,
						--			L_CURRENT_UTC_TIMESTAMP,
						L_CURRENT_UTC_TIMESTAMP,
						L_CURRENT_UTC_TIMESTAMP,
						L_CURRENT_UTC_TIMESTAMP,
						L_CURRENT_UTC_TIMESTAMP,
						I_USER_NAME,
						L_CURRENT_UTC_TIMESTAMP,
						I_USER_NAME,
						L_CURRENT_UTC_TIMESTAMP,
						I_USER_NAME,
						L_CURRENT_UTC_TIMESTAMP,
						0,
						'0'
					);
					
					VALUES IDENTITY_VAL_LOCAL()	INTO  P_ALERT_ID;
--					IF (L_SUSPEND = 'true') THEN
 
--						 INSERT INTO FCM_ALERT_ACTION (ALERT_ID ,
--									ALERT_VERSION_NBR ,
--									ACTION_TIMESTAMP ,
--									XQO_CUST_NUM ,
--									ALERT_VALUE ,
--									ALERT_TYPE ,
--									MULTI_ORG_ID, 
--									ALERT_STATUS ,
--									GMT_OFFSET ,
--									ALERT_SERVICE_CNT ,
--									ALERT_ORIGIN ,
--									CREATE_USER ,
--									CREATE_TIMESTAMP ) 
--							VALUES( P_ALERT_ID ,
--									0 ,
--									L_CURRENT_UTC_TIMESTAMP ,
--									I_CUST_NUM ,
--									I_ALERT_VALUE ,
--									l_ALERT_TYPE ,
--									L_MULTI_ORG_ID, 
--									'SUSPEND' ,
--									l_GMT_OFFSET ,
--									1,
--									'INBOUND',
--									I_USER_NAME ,
--									L_CURRENT_UTC_TIMESTAMP  );
--					END IF;					

					-- step 2a. get alert
					
					SELECT
						ALERT_ID,
						ALERT_VERSION_NBR,
						ALERT_VALUE,
						AQO_ACCT_NUM,
						XQO_CUST_NUM,
						QUEUE_ID,
						MULTI_ORG_ID,
						ALERT_TYPE,
						RESULT_ID,
						ALERT_STATUS,
						GMT_OFFSET,
						FRAUD_TYPE_CODE,
						PEND_REASON_CODE,
						FRAUD_SCORE,
						FRAUD_SCORE_REASON_CODE_1,
						FRAUD_SCORE_REASON_CODE_2,
						FRAUD_SCORE_REASON_CODE_3,
						INIT_FRAUD_SCORE,
						INIT_FRAUD_SCORE_REASON_CODE_1,
						INIT_FRAUD_SCORE_REASON_CODE_2,
						INIT_FRAUD_SCORE_REASON_CODE_3,
						HWM_FRAUD_SCORE,
						HWM_FRAUD_SCORE_REASON_CODE_1,
						HWM_FRAUD_SCORE_REASON_CODE_2,
						HWM_FRAUD_SCORE_REASON_CODE_3,
						ALERT_SERVICE_CNT,
						ALERT_ORIGIN,
						LST_TRANSACTION_AMT,
						LST_TRANSACTION_TIMESTAMP,
						USER_SERVICE_TIMESTAMP,
						SYSTEM_SERVICE_TIMESTAMP,
						NEXT_SERVICE_TIMESTAMP,
						QUEUE_ENTRY_TIMESTAMP,
						FINAL_DISPOSITION_TIMESTAMP,
						CHECKOUT_USER,
						CHECKOUT_TIMESTAMP,
						CREATE_USER,
						CREATE_TIMESTAMP,
						LSTUPDT_USER,
						LSTUPDT_TIMESTAMP,
						LSTUPDT_NBR,
						LOGICAL_DEL_IND,
						BASE_RULE_ID,
						RULE_VERSION_NBR
					INTO
						L_ALERT_ID,
						L_ALRT_VER_NBR,
						L_ALERT_VALUE,
						L_ACCT_NUM,
						L_CUST_NUM,
						L_QUEUE_ID,
						L_MULTI_ORG_ID,
						L_ALERT_TYPE,
						L_RESULT_ID ,
						L_ALERT_STATUS,
						L_GMT_OFFSET,
						L_FRAUD_TYPE_CODE ,
						L_PEND_REASON_CODE,
						L_FRAUD_SCORE ,
						L_FRAUD_SCORE_REASON_CODE_1 ,
						L_FRAUD_SCORE_REASON_CODE_2 ,
						L_FRAUD_SCORE_REASON_CODE_3 ,
						L_INIT_FRAUD_SCORE,
						L_INIT_FRAUD_SCORE_REASON_CODE_1,
						L_INIT_FRAUD_SCORE_REASON_CODE_2,
						L_INIT_FRAUD_SCORE_REASON_CODE_3,
						L_HWM_FRAUD_SCORE ,
						L_HWM_FRAUD_SCORE_REASON_CODE_1 ,
						L_HWM_FRAUD_SCORE_REASON_CODE_2 ,
						L_HWM_FRAUD_SCORE_REASON_CODE_3 ,
						L_ALERT_SERVICE_CNT ,
						L_ALERT_ORIGIN,
						L_LST_TRANSACTION_AMT ,
						L_LST_TRANSACTION_TIMESTAMP ,
						L_USER_SERVICE_TIMESTAMP,
						L_SYSTEM_SERVICE_TIMESTAMP,
						L_NEXT_SERVICE_TIMESTAMP,
						L_QUEUE_ENTRY_TIMESTAMP ,
						L_FINAL_DISPOSITION_TIMESTAMP ,
						L_CHECKOUT_USER ,
						L_CHECKOUT_TIMESTAMP,
						L_CREATE_USER ,
						L_CREATE_TIMESTAMP,
						L_LSTUPDT_USER,
						L_LSTUPDT_TIMESTAMP ,
						L_LSTUPDT_NBR ,
						L_LOGICAL_DEL_IND,
						L_BASE_RULE_ID,
						L_RULE_VERSION_NBR
					FROM FCM_ALERT
					WHERE
						ALERT_ID = P_ALERT_ID;
					
					CALL SAVE_ALERT_ACTION
										( L_ALERT_ID
											,L_ALRT_VER_NBR
											,L_ALERT_VALUE
											,L_ACCT_NUM
											,L_CUST_NUM
											,L_QUEUE_ID
											,L_MULTI_ORG_ID
											,L_ALERT_TYPE
											,L_RESULT_ID
											,L_ALERT_STATUS
											,L_GMT_OFFSET
											,L_FRAUD_TYPE_CODE
											,L_FRAUD_SCORE
											,L_FRAUD_SCORE_REASON_CODE_1
											,L_FRAUD_SCORE_REASON_CODE_2
											,L_FRAUD_SCORE_REASON_CODE_3
											,L_INIT_FRAUD_SCORE
											,L_INIT_FRAUD_SCORE_REASON_CODE_1
											,L_INIT_FRAUD_SCORE_REASON_CODE_2
											,L_INIT_FRAUD_SCORE_REASON_CODE_3
											,L_HWM_FRAUD_SCORE
											,L_HWM_FRAUD_SCORE_REASON_CODE_1
											,L_HWM_FRAUD_SCORE_REASON_CODE_2
											,L_HWM_FRAUD_SCORE_REASON_CODE_3
											,L_ALERT_SERVICE_CNT
											,L_ALERT_ORIGIN
											,L_LST_TRANSACTION_AMT
											,L_LST_TRANSACTION_TIMESTAMP
											,L_USER_SERVICE_TIMESTAMP
											,L_SYSTEM_SERVICE_TIMESTAMP
											,L_NEXT_SERVICE_TIMESTAMP
											,L_QUEUE_ENTRY_TIMESTAMP
											,L_FINAL_DISPOSITION_TIMESTAMP
											,L_CHECKOUT_USER
											,L_CHECKOUT_TIMESTAMP
											,I_USER_NAME
											,L_CREATE_TIMESTAMP
											,I_USER_NAME
											,L_LSTUPDT_TIMESTAMP
											,L_LSTUPDT_NBR
											--,NULL --,l_pend_reason_code
											,NULL --,l_reroute_queue_id
											,'INB' -- ,l_service_method_code
											,'ALERTINIT' --l_action_type
											,'N' -- l_need_to_commit
											,L_BASE_RULE_ID
											,L_RULE_VERSION_NBR
											,L_PAUSE_TIMESTAMP
											,L_PAUSE_ELAPSED_TIME
											,NULL -- ,l_strategy_id
						);
			END IF;
			END IF;
			
			-- step 4. commit if y
			IF (I_NEED_TO_COMMIT = 'Y') THEN
					COMMIT;
					CALL GET_ALERT(P_ALERT_ID, I_USER_NAME);
			END IF;

ELSE
	CALL GET_ALERT(-1, I_USER_NAME);
END IF;
--
END CRALERT @

--GRANT EXECUTE ON SPECIFIC PROCEDURE CR_MAN_ALERT TO ROLE APPROLE@