

CALL SYSPROC.SET_ROUTINE_OPTS('ISOLATION UR') @

CREATE PROCEDURE ENABLE_SUSPEND_ALERTS (
   IN I_ENABLE CHAR (1) 
   
)
   SPECIFIC ENABLE_SUSPEND_ALERTS
   LANGUAGE SQL
   NOT DETERMINISTIC
   EXTERNAL ACTION
   MODIFIES SQL DATA
   CALLED ON NULL INPUT
   INHERIT SPECIAL REGISTERS

-----------------------------------------------------------------------------
-- File:   enabl.sqe_suspend_alerts.sql
--
--
-- Description:  Procedure to change the enableing and disabling of the SUSPEND_ALERT processing
-- 
--
-- Called By: Should only be called on advice from SAS
--
--
-- Calls:
--     UNCHECKOUT_SPECIFIC_ALERT
--
-- Notes:
--
-- Usage:
--		I_ENABLE must be a '1' or '0'
--			'1' means enable alert suspending
--          '0' means disable alert suspending
--   
--
-- History:     
--              2010/04/16   LEUSHE  Created 
--	            2010/05/24   LEUSHE  Added UNSUSPEND Alert Action insert
--              2010/10/22   leushe  Changed references to GRX_, GRA_ and GRC_ to XQO_,AQO_, and blank respectively----
-- 			    2011/03/30   leushe  Commented out the insert into FCM_ALERT_ACTION for SUSPEND actions
--
------------------------------------------------------------------------
UNCHK:
BEGIN

-- step 0. declare local variables
DECLARE L_ALERT_ID          BIGINT DEFAULT -1 ;
DECLARE L_SUSPEND CHAR(5) DEFAULT 'false';
DECLARE L_STMTTEMPLATE VARCHAR(1000);
DECLARE V_SQLSTATE          CHAR (5) DEFAULT '00000' ;
DECLARE SQLSTATE            CHAR (5) DEFAULT '00000' ;
DECLARE L_STMT STATEMENT;
DECLARE C_STMT CURSOR WITH HOLD FOR L_STMT;

 --
--Check to see if suspended alerts is on
--
	SELECT coalesce(SUBSTR(LCASE(PROPERTY_VALUE),1,5),'false')   INTO L_SUSPEND FROM FCM_PROPERTY WHERE PROPERTY_NAME = 
    	'analyst.alert.suspend.enabled';
    IF (L_SUSPEND is null) THEN
    	set L_SUSPEND = 'false';
    END IF;
    IF (I_ENABLE = '0' AND L_SUSPEND = 'true') THEN
	   	UPDATE FCM_PROPERTY SET PROPERTY_VALUE = 'false',
		   	lstupdt_user = 'DEFVAL',
	    	lstupdt_timestamp = current timestamp - current timezone
	    	WHERE PROPERTY_NAME = 
	    	'analyst.alert.suspend.enabled';
	    
--	    INSERT INTO FCM_ALERT_ACTION (ALERT_ID ,
--			ALERT_VERSION_NBR ,
--			ACTION_TIMESTAMP ,
--			XQO_CUST_NUM ,
--			ALERT_VALUE ,
--			ALERT_TYPE ,
--			MULTI_ORG_ID, 
--			ALERT_STATUS ,
--			GMT_OFFSET ,
--			ALERT_SERVICE_CNT ,
--			ALERT_ORIGIN ,
--			CREATE_USER ,
--			CREATE_TIMESTAMP ) 
--			SELECT a.ALERT_ID ,
--				a.ALERT_VERSION_NBR ,
--				CURRENT TIMESTAMP - CURRENT TIMEZONE ,
--				a.XQO_CUST_NUM ,
--				a.ALERT_VALUE ,
--				a.ALERT_TYPE ,
--				a.MULTI_ORG_ID, 
--				'UNSUSPEND' ,
--				a.GMT_OFFSET ,
--				a.ALERT_SERVICE_CNT ,
--				a.ALERT_ORIGIN ,
--				a.CREATE_USER ,
--				a.CREATE_TIMESTAMP  
--			FROM FCM_ALERT a, FCM_ALERT_SUSPEND fas
--			WHERE a.ALERT_ID = fas.ALERT_ID 
--				AND A.xqo_cust_num = fas.xqo_cust_num;
	    	
	    DELETE FROM FCM_ALERT_SUSPEND; 

	 ELSEIF (I_ENABLE = '1' AND L_SUSPEND = 'false') THEN
	   	
    	UPDATE FCM_PROPERTY SET PROPERTY_VALUE = 'true',
	    	lstupdt_user = 'DEFVAL',
	    	lstupdt_timestamp = current timestamp - current timezone
	   	WHERE PROPERTY_NAME = 
	    	'analyst.alert.suspend.enabled';
    	
		SET L_STMTTEMPLATE = ' SELECT alert_id FROM fcm_alert WHERE ALERT_STATUS = ''CHECKOUT''  FOR READ ONLY';
      
	    PREPARE L_STMT FROM L_STMTTEMPLATE;
	    OPEN C_STMT ;
			  
	    SET V_SQLSTATE = SQLSTATE;
	    WHILE (V_SQLSTATE = '00000')
	      DO
	
	      FETCH C_STMT INTO L_ALERT_ID;
	      SET V_SQLSTATE = SQLSTATE;
	      IF (V_SQLSTATE = '00000')         THEN
				call UNCHECKOUT_SPECIFIC_ALERT (l_alert_id,'DEFVAL');
	      END IF;
	
	    END WHILE;
	
	END IF;	     

END UNCHK @


