

CALL SYSPROC.SET_ROUTINE_OPTS('isolation ur') @

CREATE PROCEDURE FORCE_CHECKOUT (
    IN I_USER_NAME VARCHAR(60),
    IN I_ALERT_ID BIGINT,
    IN I_LSTUPDT_NBR INTEGER,
    OUT O_USER_NAME VARCHAR(60),
    OUT O_SQLCODE INTEGER,
    OUT O_SQLSTATE CHAR(5) )
SPECIFIC FORCE_CHECKOUT
DYNAMIC RESULT SETS 1
LANGUAGE SQL
-----------------------------------------------------------------------------
-- file:  force_checkout
--
--
-- description:
--    allows the app to handle a case where an analyst is working an alert
--    but an inbound call is received by another analyst for that same
--    account.  in this case, the analyst handling the inbound call trumps
--    the original analyst and steals the alert from the former.
--    The procedure assumes the User has access by Multi_org and Alert Type
--    To checkout this alert. 
--
--
-- calls:
--	   get_alert_row
--     get_alert
--     save_alert_action
--
-- notes:
--
--
-- usage:
--		Called from the web UI
--
--
-- history:
--       2005/12/06    jicimi      taken from checkout_and_get_alert
--       2006/03/28    jicimi      added action_type parameter to save_alert_action call
--       2006/04/25    jicimi      l_fraud_type_code from char(2) to char(3)
--       2006/05/26    jicimi      dropped alert_trigger_values
--                                 added base_rule_id & rule_version_nbr
--       2006/06/05    jicimi      result_id set to null in the alert action row
--                                 added pause_ts = null & elapsed_pause_time = 0
--                                    to the alert update statement
--       2006/10/05    jicimi      if i_user_name is the same as the checkout_user
--                                    then don't audit the force checkout
--       2007/02/12    jicimi      added strategy_id
--       2007/06/19    sas         adapted for r2
--       2007/10/01                save alert action changes ( acct_num & cust_num )
--   	 2008/11/20    leushe      included alert_level column
--	     2009/02/05    leushe      included the alert_version_nbr column, which is a new column in v3
--	     2009/06/22    leushe      increased lenghts of alert_value, cust_num,acct_num, and user_name
--       2009/08/14    leushe      Added the security check to ensure the user_name has access to update the alert
--       2010/04/14    leushe      Updated the call signature to save_alert_action to not send pend_reason
--       2010/04/26    leushe      Checks for alert suspension mode, and ensures it can update the FCM_ALERT_SUSPEND record 
--                                 prior to checkout
--       2010/10/22    leushe      Changed references to GRX_, GRA_ and GRC_ to XQO_,AQO_, and blank respectively--
--       2011/04/01    leushe      Changed the FCM_ALERT_SUSPEND to fix race condition and addition of ALERT_CHECKOUT_CNT column
------------------------------------------------------------------------
P1: BEGIN

    DECLARE L_ROWCOUNT INTEGER DEFAULT -1;
    DECLARE APPSQLEXC_UPDATESINGLEROW    CONDITION FOR SQLSTATE '99002';
    DECLARE SQLCODE INTEGER DEFAULT 0;
    DECLARE SQLSTATE CHAR(5) DEFAULT '00000';

    DECLARE L_NEW_ALERT_STATUS CHARACTER(10);

    DECLARE   L_ALERT_ID BIGINT;
    DECLARE   L_ALRT_VER_NBR INTEGER;
    DECLARE   L_ALERT_VALUE  CHARACTER(34) ;
    DECLARE   L_ACCT_NUM  CHARACTER(34) ;
    DECLARE   L_CUST_NUM  CHARACTER(34) ;
    DECLARE   L_QUEUE_ID BIGINT ;
    DECLARE   L_MULTI_ORG_ID INTEGER;
    DECLARE   L_ALERT_TYPE CHAR(1);
    DECLARE   L_RESULT_ID  INTEGER;
    DECLARE   L_ALERT_STATUS CHARACTER(10) ;
    DECLARE   L_GMT_OFFSET DECIMAL(3,1) DEFAULT 0.0;
    DECLARE   L_FRAUD_TYPE_CODE  CHARACTER(3);
    DECLARE   L_PEND_REASON_CODE CHARACTER(4);
    DECLARE   L_FRAUD_SCORE  INTEGER;
    DECLARE   L_FRAUD_SCORE_REASON_CODE_1  INTEGER;
    DECLARE   L_FRAUD_SCORE_REASON_CODE_2   INTEGER;
    DECLARE   L_FRAUD_SCORE_REASON_CODE_3   INTEGER;
    DECLARE   L_INIT_FRAUD_SCORE INTEGER;
    DECLARE   L_INIT_FRAUD_SCORE_REASON_CODE_1  INTEGER;
    DECLARE   L_INIT_FRAUD_SCORE_REASON_CODE_2  INTEGER;
    DECLARE   L_INIT_FRAUD_SCORE_REASON_CODE_3  INTEGER;
    DECLARE   L_HWM_FRAUD_SCORE  INTEGER;
    DECLARE   L_HWM_FRAUD_SCORE_REASON_CODE_1   INTEGER;
    DECLARE   L_HWM_FRAUD_SCORE_REASON_CODE_2   INTEGER;
    DECLARE   L_HWM_FRAUD_SCORE_REASON_CODE_3   INTEGER;
    DECLARE   L_ALERT_SERVICE_CNT  INTEGER  ;
    DECLARE   L_ALERT_ORIGIN VARCHAR(20) ;
    DECLARE   L_LST_TRANSACTION_AMT  DECIMAL(19,5);
    DECLARE   L_LST_TRANSACTION_TIMESTAMP  TIMESTAMP;
    DECLARE   L_USER_SERVICE_TIMESTAMP TIMESTAMP;
    DECLARE   L_SYSTEM_SERVICE_TIMESTAMP TIMESTAMP;
    DECLARE   L_NEXT_SERVICE_TIMESTAMP TIMESTAMP;
    DECLARE   L_QUEUE_ENTRY_TIMESTAMP  TIMESTAMP;
    DECLARE   L_FINAL_DISPOSITION_TIMESTAMP  TIMESTAMP;
    DECLARE   L_CHECKOUT_USER  CHARACTER(60) ;
    DECLARE   L_CHECKOUT_TIMESTAMP TIMESTAMP;
    DECLARE   L_CREATE_USER  CHARACTER(60) ;
    DECLARE   L_CREATE_TIMESTAMP TIMESTAMP;
    DECLARE   L_LSTUPDT_USER CHARACTER(60);
    DECLARE   L_LSTUPDT_TIMESTAMP  TIMESTAMP;
    DECLARE   L_LSTUPDT_NBR  INTEGER;
    DECLARE   L_BASE_RULE_ID BIGINT;
    DECLARE   L_RULE_VERSION_NBR INTEGER;
    DECLARE L_RESCHEDULE_MINUTES INTEGER;
    DECLARE L_SERVICE_METHOD_CODE CHAR(4);
    DECLARE L_LOGICAL_DEL_IND  CHARACTER(1);
  --  declare l_former_checkout_user  character(60);
    DECLARE L_ACTION_TYPE CHARACTER(20) DEFAULT 'analyst action';
    DECLARE L_PAUSE_TIMESTAMP TIMESTAMP;
    DECLARE L_PAUSE_ELAPSED_TIME INTEGER;
    DECLARE L_SUSPEND CHAR(5) DEFAULT 'false';

    --
    --  logic overview...
    --
    --  1. Check to ensure the user has access to ther alert_id
    --  2. get the requested alert - returning it into local host variables
    --  3. save anlyst's user_name who is currently responsible for resolving the alert
    --  4. steal the alert ->
    --        update the alert in the alert table status='transfer', service_method, user_name, ....
    --  5. insert stolen alert row into action table
    --  6. commit;
    --  7. return current alert row and analyst's user_name to the client
    --  8. ?! ui notify the former owner to move on to another alert
    --

IF (CHECK_ALERT_ACCESS(I_USER_NAME,I_ALERT_ID) = 1) THEN

    /* 1 */
    CALL GET_ALERT_ROW
         ( I_ALERT_ID
          ,L_ALRT_VER_NBR
          ,L_ALERT_VALUE
          ,L_ACCT_NUM
          ,L_CUST_NUM
          ,L_QUEUE_ID
          ,L_MULTI_ORG_ID
          ,L_ALERT_TYPE
          ,L_RESULT_ID
          ,L_ALERT_STATUS
          ,L_GMT_OFFSET
          ,L_FRAUD_TYPE_CODE
          ,L_FRAUD_SCORE
          ,L_FRAUD_SCORE_REASON_CODE_1
          ,L_FRAUD_SCORE_REASON_CODE_2
          ,L_FRAUD_SCORE_REASON_CODE_3
          ,L_INIT_FRAUD_SCORE
          ,L_INIT_FRAUD_SCORE_REASON_CODE_1
          ,L_INIT_FRAUD_SCORE_REASON_CODE_2
          ,L_INIT_FRAUD_SCORE_REASON_CODE_3
          ,L_HWM_FRAUD_SCORE
          ,L_HWM_FRAUD_SCORE_REASON_CODE_1
          ,L_HWM_FRAUD_SCORE_REASON_CODE_2
          ,L_HWM_FRAUD_SCORE_REASON_CODE_3
          ,L_ALERT_SERVICE_CNT
          ,L_ALERT_ORIGIN
          ,L_LST_TRANSACTION_AMT
          ,L_LST_TRANSACTION_TIMESTAMP
          ,L_USER_SERVICE_TIMESTAMP
          ,L_SYSTEM_SERVICE_TIMESTAMP
          ,L_NEXT_SERVICE_TIMESTAMP
          ,L_QUEUE_ENTRY_TIMESTAMP
          ,L_FINAL_DISPOSITION_TIMESTAMP
          ,L_CHECKOUT_USER
          ,L_CHECKOUT_TIMESTAMP
          ,L_CREATE_USER
          ,L_CREATE_TIMESTAMP
          ,L_LSTUPDT_USER
          ,L_LSTUPDT_TIMESTAMP
          ,L_LSTUPDT_NBR
          ,L_BASE_RULE_ID
          ,L_RULE_VERSION_NBR
          ,L_PAUSE_TIMESTAMP
          ,L_PAUSE_ELAPSED_TIME
          );



    /* 2 & 6 */
   -- set l_former_checkout_user = l_checkout_user;
    SET O_USER_NAME = L_CHECKOUT_USER;


    IF ( L_ALERT_STATUS = 'CHECKOUT' OR L_ALERT_STATUS = 'TRANSFER' ) THEN
       SET L_NEW_ALERT_STATUS = 'TRANSFER';
       --set l_new_alert_status = 'STOLEN';
    ELSE
       SET L_NEW_ALERT_STATUS = 'CHECKOUT';
    END IF;


    
 --
--Check to see if suspended alerts is on
--
SELECT coalesce(SUBSTR(LCASE(PROPERTY_VALUE),1,5),'false')   INTO L_SUSPEND FROM FCM_PROPERTY WHERE PROPERTY_NAME = 
    	'analyst.alert.suspend.enabled';
    IF (L_SUSPEND is null) THEN
    	set L_SUSPEND = 'false';
    END IF;
    
    IF (L_SUSPEND = 'true' and I_USER_NAME != L_CHECKOUT_USER ) THEN
    	UPDATE FCM_ALERT_SUSPEND SET 
    			SUSPEND_USER = I_USER_NAME,
    			SUSPEND_TIMESTAMP= CURRENT TIMESTAMP - CURRENT TIMEZONE
    	WHERE 
    		xqo_cust_num = L_CUST_NUM
    		AND ALERT_CHECKOUT_CNT = 1;


    	GET DIAGNOSTICS L_ROWCOUNT = ROW_COUNT;

    ELSE
	 SET L_ROWCOUNT = 1;
	
    END IF;
    /* 3 */
    If(L_SUSPEND = 'false' or L_ROWCOUNT  = 1) THEN
    UPDATE FCM_ALERT
      SET LSTUPDT_USER = I_USER_NAME,
          -- ui can't handle a status of transfer
          ALERT_STATUS = 'CHECKOUT',
          --alert_status = l_new_alert_status,
          -- was the alert really serviced by the engine?
          ALERT_SERVICE_CNT = ALERT_SERVICE_CNT + 1,
          CHECKOUT_USER = I_USER_NAME,
          USER_SERVICE_TIMESTAMP = CURRENT TIMESTAMP - CURRENT TIMEZONE,
          LSTUPDT_TIMESTAMP = CURRENT TIMESTAMP - CURRENT TIMEZONE,
          LSTUPDT_NBR = LSTUPDT_NBR + 1,
          CHECKOUT_TIMESTAMP = CURRENT TIMESTAMP - CURRENT TIMEZONE,
          PAUSE_TIMESTAMP = NULL,
          PAUSE_ELAPSED_TIME = 0
    WHERE ALERT_ID = I_ALERT_ID
      AND LSTUPDT_NBR = I_LSTUPDT_NBR
    ;
   
    GET DIAGNOSTICS L_ROWCOUNT = ROW_COUNT;
    END IF;
    
    IF ( L_ROWCOUNT = 1 ) THEN


       --
       -- 2006/10/05
       --   if the user name is the same as
       --   the old user don't audit anything
       --
       IF ( I_USER_NAME != L_CHECKOUT_USER ) THEN

       /* 4 */
       CALL SAVE_ALERT_ACTION
            ( I_ALERT_ID
             ,L_ALRT_VER_NBR
             ,L_ALERT_VALUE
             ,L_ACCT_NUM
             ,L_CUST_NUM
             ,L_QUEUE_ID
             ,L_MULTI_ORG_ID
             ,L_ALERT_TYPE
             ,NULL                 -- 2006/06/05 change  ,l_result_id
             ,L_NEW_ALERT_STATUS   -- ,l_alert_status
             ,L_GMT_OFFSET
             ,L_FRAUD_TYPE_CODE
             ,L_FRAUD_SCORE
             ,L_FRAUD_SCORE_REASON_CODE_1
             ,L_FRAUD_SCORE_REASON_CODE_2
             ,L_FRAUD_SCORE_REASON_CODE_3
             ,L_INIT_FRAUD_SCORE
             ,L_INIT_FRAUD_SCORE_REASON_CODE_1
             ,L_INIT_FRAUD_SCORE_REASON_CODE_2
             ,L_INIT_FRAUD_SCORE_REASON_CODE_3
             ,L_HWM_FRAUD_SCORE
             ,L_HWM_FRAUD_SCORE_REASON_CODE_1
             ,L_HWM_FRAUD_SCORE_REASON_CODE_2
             ,L_HWM_FRAUD_SCORE_REASON_CODE_3
             ,L_ALERT_SERVICE_CNT
             ,L_ALERT_ORIGIN
             ,L_LST_TRANSACTION_AMT
             ,L_LST_TRANSACTION_TIMESTAMP
             ,L_USER_SERVICE_TIMESTAMP
             ,L_SYSTEM_SERVICE_TIMESTAMP
             ,L_NEXT_SERVICE_TIMESTAMP
             ,L_QUEUE_ENTRY_TIMESTAMP
             ,L_FINAL_DISPOSITION_TIMESTAMP
             ,I_USER_NAME --,l_checkout_user
             ,L_CHECKOUT_TIMESTAMP
             ,L_CREATE_USER
             ,L_CREATE_TIMESTAMP
             ,I_USER_NAME --,l_lstupdt_user
             ,L_LSTUPDT_TIMESTAMP
             ,L_LSTUPDT_NBR
          --   ,null --,l_pend_reason_code
             ,NULL --,l_reroute_queue_id
             ,'INB'     -- inbound   ,l_service_method_code
             ,L_ACTION_TYPE
             ,'Y' -- l_need_to_commit
             ,L_BASE_RULE_ID
             ,L_RULE_VERSION_NBR
             ,L_PAUSE_TIMESTAMP
             ,L_PAUSE_ELAPSED_TIME
             ,NULL -- strategy_id
             );

       END IF;

       /* 5 & 6 */
       COMMIT;
       SET O_SQLCODE = SQLCODE;
       SET O_SQLSTATE = SQLSTATE;
       CALL GET_ALERT(I_ALERT_ID, I_USER_NAME);


    ELSE
       -- eventually will need to signal an exception; now just return an empty set
       --signal appsqlexc_updatesinglerow set message_text = 'failure: could not steal alert.';
       ROLLBACK;
       SET O_SQLCODE = -1;
       SET O_SQLSTATE = 'ERROR';
       CALL GET_ALERT(-1, I_USER_NAME);
    END IF;
ELSE
       -- User does not have access to the alert, so send blank result set back
       SET O_SQLCODE = -1;
       SET O_SQLSTATE = 'PMERR';
       CALL GET_ALERT(-1, I_USER_NAME);
END IF;

END P1 @

--GRANT EXECUTE ON SPECIFIC PROCEDURE FORCE_CHECKOUT TO ROLE APPROLE @