
call sysproc.set_routine_opts('ISOLATION UR') @

CREATE PROCEDURE GET_ALERT_FROM_QUEUE ( IN i_user_id bigint,
                                        IN i_user_name CHAR(60),
                                        IN i_queue_id bigint,
                                        IN i_queue_sort_field VARCHAR(30),
                                        IN i_queue_sort_order VARCHAR(10),
                                       
                                      OUT o_alert_id BIGINT )
SPECIFIC GET_ALERT_FROM_QUE
DYNAMIC RESULT SETS 0
LANGUAGE SQL
-----------------------------------------------------------------------------
-- File:  GET_ALERT_FROM_QUE
--
--
-- Description:
--
--
-- Called By:
--     qry_alert_service
--
-- Calls:
--
--
-- Notes:
--
--
-- Usage:
--
--
-- History:    10/17/2005:          taken from FCMINT and stored in PRISM
--             2006/03/30   jicimi  changed -5 literal ( acct_gmt_offest )
--                                  to -8 (PST)
--             2006/04/18   jicimi  l_baseSql changed from varchar(2000) to
--                                  varchar(2500)
--             2006/04/17   jicimi  changed -8 literal ( acct_gmt_offest ) to
--                                  the value of fcm_alert.account_gmt_offset
--             2006/06/05   jicimi  added cms_acct_gmt_offset
--                                  update to fcm_alert sets pause_time = NULL
--                                    & pause_elapsed_time = 0
--                                  added logical_del_ind checks
--             2006/08/02           added alert_status ACTIVE check to
--                                    the predicate of the update statement
--                                  added the is null predicate to first subselect
--                                     in the odd event that the rank criterion
--                                     doesn't have a value
--
-- 			   2007/06/18           add curr_cust_hour UDF
--             18DEC2008    LEUSHE  S0553598 Removed checking on the SQLSTATE variable. Changed this by setting 
--                                  the v_sqlstate variable SQLSTATE, the conditionally checking v_sqlstate.
--                                  In db2 9.5, accessing the SQLSTATE variable, resets it. Thus, the IF would reset the SQLSTATE 
--                                  value. 
-- 			   2009/06/10           Updated for V3
--             2009/08/14    leushe Added the security check to ensure the user_name has access to update the alert
--             2010/04/26    leushe Added logic to return only alerts that are not suspended, and when gets an alert,  
--                                  adds the alert to the FCM_ALERT_SUSPEND table
--      	   2010/05/24    leushe Added SUSPEND Alert Action insert
--             2010/10/22    leushe Changed references to GRX_, GRA_ and GRC_ to XQO_,AQO_, and blank respectively--
-- 			   2011/03/28    leushe Changed the dynamic statement query for performance
-- 			   2011/03/30    leushe Commented out the insert into alert action for suspend record
--             2011/04/01    leushe Changed the FCM_ALERT_SUSPEND to fix race condition and addition of ALERT_CHECKOUT_CNT column
------------------------------------------------------------------------

P1: BEGIN

    DECLARE l_alert_id bigint DEFAULT -1;
    DECLARE l_row_count INT DEFAULT 0;

    DECLARE v_sqlstate CHAR(5) DEFAULT '00000';
    DECLARE SQLSTATE CHAR(5) DEFAULT '00000';
    DECLARE l_sort char(4) DEFAULT 'desc';

    DECLARE l_baseSql varchar(2500);
    DECLARE l_suspend char(5);
    DECLARE l_cust_num char(34);

    DECLARE l_ALERT_VERSION_NBR INTEGER;
    DECLARE l_ALERT_VALUE CHAR(34);
    DECLARE l_ALERT_TYPE CHAR(1);
    DECLARE l_MULTI_ORG_ID INTEGER;
    DECLARE l_GMT_OFFSET DECIMAL(3,1);
    DECLARE l_ALERT_SERVICE_CNT INTEGER;
    DECLARE l_ALERT_ORIGIN VARCHAR(20);
    DECLARE l_CREATE_USER CHAR(60);
    DECLARE l_CREATE_TIMESTAMP TIMESTAMP;
    
    DECLARE l_stmt STATEMENT;
    
    DECLARE c_stmt CURSOR for l_stmt;

    DECLARE CONTINUE HANDLER FOR SQLSTATE  '23505'
    	BEGIN
        set l_row_count = -20;
      END;
    /* set not used, just needed something there */
--
--Check to see if suspended alerts is on
--
SELECT coalesce(SUBSTR(LCASE(PROPERTY_VALUE),1,5),'false')   INTO L_SUSPEND FROM FCM_PROPERTY WHERE PROPERTY_NAME = 
    	'analyst.alert.suspend.enabled';

    IF (L_SUSPEND is null) THEN
    	set L_SUSPEND = 'false';
    END IF;
    
    IF ( i_queue_sort_order = 'ASC' ) THEN
       SET l_sort = ' ';
    END IF;

--	SET l_baseSql =
--			'select alert_id,xqo_cust_num, ALERT_VERSION_NBR ,ALERT_VALUE ,ALERT_TYPE ,
--			MULTI_ORG_ID, GMT_OFFSET , ALERT_SERVICE_CNT ,ALERT_ORIGIN ,aa.CREATE_USER ,
--			aa.CREATE_TIMESTAMP
--               from fcm_alert aa, 
--               		fcm_queue q
--               where aa.queue_id = ? and
--                     check_mo_at_access( cast ( ? as BIGINT) ,aa.multi_org_id,aa.alert_type)=1 and
--                      aa.queue_id = q.queue_id and
--                     aa.alert_status = ''ACTIVE'' and
--                     (aa.next_service_timestamp) <  (current timestamp - current timezone) and
--                     (
--                         (q.queue_call_superseded_ind = ''1'') or
--                         ( curr_cust_hour( aa.gmt_offset) between
--                                    q.queue_call_lower_bound and q.queue_call_upper_bound )
--                      )
-- 					and not exists (select 1 from fcm_alert_suspend 
--						where xqo_cust_num = aa.xqo_cust_num 
--						      and suspend_user <> ? )
--                order by aa.'||i_queue_sort_field||' '||l_sort||'
--                fetch first 10 rows only
--                for read only
--                optimize for 1 row /*with UR*/';
	SET l_baseSql = 'select alert_id, 
       xqo_cust_num, 
       ALERT_VERSION_NBR, 
       ALERT_VALUE, 
       ALERT_TYPE, 
       MULTI_ORG_ID, 
       GMT_OFFSET, 
       ALERT_SERVICE_CNT, 
       ALERT_ORIGIN, 
       aa.CREATE_USER, 
       aa.CREATE_TIMESTAMP 
  from fcm_alert aa, 
       TABLE (SELECT Q2.queue_id queue_id, 
       Q2.queue_call_superseded_ind queue_call_superseded_ind, 
       Q2.queue_call_lower_bound queue_call_lower_bound, 
       Q2.queue_call_upper_bound queue_call_upper_bound 
  FROM fcm_queue Q2 
  where q2.queue_id = ?
 ORDER BY Q2.QUEUE_ID) Q1 
 where aa.queue_id = ? 
   and check_mo_at_access(cast(? as BIGINT), aa.multi_org_id, aa.alert_type) = 1 
   and aa.queue_id = Q1.queue_id 
   and aa.alert_status = ''ACTIVE'' 
   and aa.next_service_timestamp < current timestamp - current timezone 
   and (Q1.queue_call_superseded_ind = ''1''
         or curr_cust_hour(aa.gmt_offset) between Q1.queue_call_lower_bound and Q1.queue_call_upper_bound) 
   and NOT exists (select 1 
  from fcm_alert_suspend 
 where xqo_cust_num = aa.xqo_cust_num 
   and suspend_user <> ?
    ) 
  order by aa.'||i_queue_sort_field||' '||l_sort||'
 fetch first 10 rows only 
   for read only';


		PREPARE l_stmt FROM l_baseSql;
		OPEN c_stmt using i_queue_id,i_queue_id,i_user_id,i_user_name;
		fetch from c_stmt into l_alert_id,l_cust_num,
		l_ALERT_VERSION_NBR ,
			l_ALERT_VALUE ,
			l_ALERT_TYPE ,
			l_MULTI_ORG_ID, 
			l_GMT_OFFSET ,
			l_ALERT_SERVICE_CNT ,
			l_ALERT_ORIGIN ,
			l_CREATE_USER ,
			l_CREATE_TIMESTAMP;
		SET v_sqlstate = SQLSTATE;
    while(v_sqlstate = '00000') do
    

-- Security check for the user should have been done in the l_baseSql statement, so no need
-- to check security on this update
	    if (l_suspend = 'true') then
        insert into fcm_alert_suspend (xqo_cust_num,alert_id,suspend_user,suspend_timestamp)
		    select l_cust_num,alert_id,i_user_name,current timestamp - current timezone 
			  from fcm_alert
			  where not exists (select 1 from fcm_alert_suspend 
						where xqo_cust_num = l_cust_num 
/* commented out for race condition*/
--							  and suspend_user <> i_user_name 
--							  and alert_id <> l_alert_id
							  )
            			and (next_service_timestamp) <  (current timestamp - current timezone)
						and alert_id = l_alert_id 
						and alert_status='ACTIVE';
 
		
		    GET DIAGNOSTICS l_row_count = ROW_COUNT;
		    
		    /* If the insert fails, then the update should fail also */
			IF(L_ROW_COUNT = 0) THEN
				UPDATE FCM_ALERT_SUSPEND
				  SET ALERT_CHECKOUT_CNT = ALERT_CHECKOUT_CNT+1
				WHERE XQO_CUST_NUM = L_CUST_NUM AND SUSPEND_USER = I_USER_NAME;
				
				GET DIAGNOSTICS L_ROW_COUNT = ROW_COUNT;
			END IF;
		        
   	     if(l_row_count = 1) then
					
--          INSERT INTO FCM_ALERT_ACTION (ALERT_ID ,
--            ALERT_VERSION_NBR ,
--            ACTION_TIMESTAMP ,
--            XQO_CUST_NUM ,
--            ALERT_VALUE ,
--            ALERT_TYPE ,
--            MULTI_ORG_ID, 
--          	ALERT_STATUS ,
--          	ACTION_TYPE ,
--          	GMT_OFFSET ,
--          	ALERT_SERVICE_CNT ,
--          	ALERT_ORIGIN ,
--          	CREATE_USER ,
--            CREATE_TIMESTAMP ) values (
--          		l_alert_id,		
--          		l_ALERT_VERSION_NBR ,
--          		CURRENT TIMESTAMP - CURRENT TIMEZONE ,
--          		l_cust_num,
--            	l_ALERT_VALUE ,
--          		l_ALERT_TYPE ,
--          		l_MULTI_ORG_ID, 
--          		'SUSPEND' ,  --alert_status
--          		'ANALYST ACTION' ,  --action_type
--          		l_GMT_OFFSET ,
--          		l_ALERT_SERVICE_CNT ,
--          		l_ALERT_ORIGIN ,
--          		l_CREATE_USER ,
--		    	l_CREATE_TIMESTAMP  );
--		
            update fcm_alert set lstupdt_user = i_user_name,
                    alert_status = 'CHECKOUT',
                    alert_service_cnt = alert_service_cnt + 1,
                    checkout_user = i_user_name,
                    checkout_timestamp =  CURRENT TIMESTAMP - CURRENT TIMEZONE,
                    user_service_timestamp = CURRENT TIMESTAMP - CURRENT TIMEZONE,
                    lstupdt_timestamp = CURRENT TIMESTAMP - CURRENT TIMEZONE,
                    lstupdt_nbr = lstupdt_nbr + 1,
                    pause_timestamp = NULL,
                    pause_elapsed_time = 0
                where
                    alert_status = 'ACTIVE' and
                    alert_id = l_alert_id
                    
                /*with UR*/;

            GET DIAGNOSTICS l_row_count = ROW_COUNT;
            if(l_row_count = 1) then
              SET v_sqlstate = '00001';
            else
              set l_alert_id = -1;
              fetch from c_stmt into l_alert_id,l_cust_num,
		l_ALERT_VERSION_NBR ,
			l_ALERT_VALUE ,
			l_ALERT_TYPE ,
			l_MULTI_ORG_ID, 
			l_GMT_OFFSET ,
			l_ALERT_SERVICE_CNT ,
			l_ALERT_ORIGIN ,
			l_CREATE_USER ,
			l_CREATE_TIMESTAMP;
      
                 SET v_sqlstate = SQLSTATE;
            end if;
		  else
   				set l_alert_id = -1;        
				fetch from c_stmt into l_alert_id,l_cust_num,
		l_ALERT_VERSION_NBR ,
			l_ALERT_VALUE ,
			l_ALERT_TYPE ,
			l_MULTI_ORG_ID, 
			l_GMT_OFFSET ,
			l_ALERT_SERVICE_CNT ,
			l_ALERT_ORIGIN ,
			l_CREATE_USER ,
			l_CREATE_TIMESTAMP;
        SET v_sqlstate = SQLSTATE;
			end if;
		else
			update fcm_alert set lstupdt_user = i_user_name,
              alert_status = 'CHECKOUT',
              alert_service_cnt = alert_service_cnt + 1,
              checkout_user = i_user_name,
              checkout_timestamp =  CURRENT TIMESTAMP - CURRENT TIMEZONE,
              user_service_timestamp = CURRENT TIMESTAMP - CURRENT TIMEZONE,
              lstupdt_timestamp = CURRENT TIMESTAMP - CURRENT TIMEZONE,
              lstupdt_nbr = lstupdt_nbr + 1,
              pause_timestamp = NULL,
              pause_elapsed_time = 0
          where
              alert_status = 'ACTIVE' and
              alert_id = l_alert_id
              
          /*with UR*/;

			GET DIAGNOSTICS l_row_count = ROW_COUNT;

			if(l_row_count = 1) then
				SET v_sqlstate = '00001';
			else
				set l_alert_id = -1;
				fetch from c_stmt into l_alert_id,l_cust_num,
			l_ALERT_VERSION_NBR ,
			l_ALERT_VALUE ,
			l_ALERT_TYPE ,
			l_MULTI_ORG_ID, 
			l_GMT_OFFSET ,
			l_ALERT_SERVICE_CNT ,
			l_ALERT_ORIGIN ,
			l_CREATE_USER ,
			l_CREATE_TIMESTAMP;
         		SET v_sqlstate = SQLSTATE;
			end if;
		end if;
		end while;
		close c_stmt;

    /*
       regardless of success ( alert_id > 0) or failure ( alert_id = -1 )
       commit to release all resources  - necessary because of the select for update
    */
    COMMIT;



    set o_alert_id = l_alert_id;

END P1@