

call sysproc.set_routine_opts('ISOLATION UR') @

CREATE PROCEDURE QRY_ALERT_SERVICE (
    IN I_USER_ID BIGINT,
    IN I_USER_NAME	VARCHAR(60),
    IN I_STRATEGY_ID	BIGINT,
    OUT O_ALERT_ID	BIGINT )
  SPECIFIC QRY_ALERT_SERVICE
  LANGUAGE SQL
--  NOT DETERMINISTIC
--  EXTERNAL ACTION
--  MODIFIES SQL DATA
--  CALLED ON NULL INPUT
--  INHERIT SPECIAL REGISTERS
-----------------------------------------------------------------------------
-- File:  QRY_ALERT_SERVICE
--
--
-- Description:
--
--
-- Called By:
--     GET_NEXT_ALERT
--
-- Calls:
--     GET_ALERT_FROM_QUEUE
--
-- Notes:     
--            This is the R3 version of qry_alert_service.
--
--
-- Usage:
--
--
-- History:    10/17/2005:  taken from FCMINT and stored in PRISM
--             2006/01/30   changed -5 literal ( acct_gmt_offest )
--                          for the actual value stored in the alert
--                          table
--             2006/02/02   l_rowcount initialized to 0
--             2006/02/09   added queue_status in active or quiesce check
--             2006/06/06   added logical_del_ind checks
--	           2007/06/18   added curr_cust_hour UDF
--	           2007/10/30   rewritten for R2
--	           2009/06/10   rewritten for R3
--             2009/08/14   leushe      Added the security check to ensure the user_name has access to update the alert
--             2010/04/26   leushe      Changed to only bring back queues that have alerts available that aren't suspended 
--                                      in the FCM_ALERT_SUSPEND table
--             2010/10/22  leushe      Changed references to GRX_, GRA_ and GRC_ to XQO_,AQO_, and blank respectively
--             2010/10/26  leushe      Added ROWNUM = 1 to the inner query. required DB2 registry setting DB2_COMPATIBILITY_VECTOR=01 
--             2011/03/28  leushe      Replaced ROWNUM with fetch first 1 row because of inconsistencies
--             2011/04/01    leushe Changed the FCM_ALERT_SUSPEND to fix race condition and addition of ALERT_CHECKOUT_CNT column
--          
--
------------------------------------------------------------------------

P1: BEGIN
  DECLARE l_index INT;
  DECLARE ii INT;
  DECLARE l_alert_id BIGINT;
  DECLARE l_rowcount INT DEFAULT 0;
  DECLARE l_rowindex INT;
  DECLARE l_queue_id BIGINT;
  DECLARE l_queue_sort_field VARCHAR(30);
  DECLARE l_queue_sort_order VARCHAR(10);

  DECLARE cursor1 CURSOR  FOR
     select q.queue_id, q.queue_sort_order, q.queue_sort_field
       from fcm_queue q, fcm_strategy_user su
       where q.strategy_id = i_strategy_id and
             q.queue_status in ('ACTIVE', 'QUIESCE') and
             su.user_id = i_user_id and
             su.strategy_id = q.strategy_id and
             q.logical_del_ind = '0' and
             queue_id =
             any (
                select a.queue_id
                from fcm_alert a
                where a.queue_id = q.queue_id and
                      check_mo_at_access(i_user_id,a.multi_org_id,a.alert_type)=1 and
                      a.alert_status='ACTIVE' and
                      a.next_service_timestamp < current timestamp - current timezone and
                      ( (  curr_cust_hour(a.gmt_offset ) between
                      q.queue_call_lower_bound and q.queue_call_upper_bound ) or
                      (q.queue_call_superseded_ind = '1') )
                      and not exists (select 1 from fcm_alert_suspend fas 
                        where fas.xqo_cust_num = a.xqo_cust_num and
                        fas.suspend_user <> i_user_name)
                        --and rownum = 1
                        fetch first 1 rows only
             )
             

             order by q.queue_priority asc
             for read only
             optimize for 1 row 
             --fetch first row only
             ;


   

  

   open cursor1;

   FETCH from cursor1 into
          l_queue_id, l_queue_sort_order, l_queue_sort_field;

   close cursor1;

   IF ( l_queue_id is NULL ) THEN -- no queues left in strategy
      set o_alert_id = -2;
      RETURN;
   END IF;
    
   -- returns -1 if no alert found available for servicing in the queue
   CALL GET_ALERT_FROM_QUEUE (I_USER_ID,i_user_name, l_queue_id, l_queue_sort_field, l_queue_sort_order, l_alert_id);


   set o_alert_id = l_alert_id;

END P1
@


