
DROP SPECIFIC PROCEDURE SAVE_ALERT2@

CALL SYSPROC.SET_ROUTINE_OPTS ('ISOLATION UR')@



CREATE PROCEDURE SAVE_ALERT (
    IN I_ALERT_ID	BIGINT,
    IN I_USER_NAME	CHARACTER(60),
    IN I_RESULT_ID	INTEGER,
    IN I_SERVICE_METHOD_CODE	CHARACTER(4),
    IN I_FRAUD_TYPE_CODE	CHARACTER(3),
    IN I_REROUTE_QUEUE_ID	BIGINT,
    IN I_RESURFACE_TIMESTAMP	TIMESTAMP,
    IN I_SYS_BLOCK_CODE_1_TS	TIMESTAMP,
    IN I_SYS_BLOCK_CODE_1	CHARACTER(8),
    IN I_SYS_BLOCK_1_UNTIL_ACTION	CHARACTER(2),
    IN I_PREVIOUS_SYS_BLOCK_CODE_1	CHARACTER(8),
    IN I_GROUP_CODE	CHARACTER(2),
    IN I_TRANS_BLOCK_TS	TIMESTAMP,
    IN I_TRANS_BLOCK_CODE	CHARACTER(8),
    IN I_TRANS_BLOCK_UNTIL_ACTION	CHARACTER(2),
    IN I_NEXT_SERVICE_TIMESTAMP	TIMESTAMP,
    IN I_CONTACT_HOME_PHONE_IND	CHARACTER(1),
    IN I_CONTACT_WORK_PHONE_IND	CHARACTER(1),
    IN I_CONTACT_MOBILE_PHONE_IND	CHARACTER(1),
    IN I_SENT_SECURE_MESSAGE_IND	CHARACTER(1),
    IN I_INVALID_CONTACT_DETAILS_IND	CHARACTER(1),
    IN I_SENT_LETTER_IND	CHARACTER(1),
    IN I_SENT_EMAIL_IND	CHARACTER(1),
    IN I_SENT_SMS_IND	CHARACTER(1),
    IN I_MEMO_TEXT	VARCHAR(1000),
    IN I_NEED_TO_COMMIT	CHARACTER(1),
    IN I_LSTUPDT_NBR	INTEGER,
    IN I_STRATEGY_ID	BIGINT,
    IN I_FULFILL_SENT_CODE	CHARACTER(1),
    IN I_FULFILL_XML_MSG	VARCHAR(10000),
    IN I_OPEN_TO_BUY_AMT	DECIMAL(19,5),
    OUT O_ALERT_ACTION_ID	BIGINT,
    OUT O_SQLCODE	INTEGER,
    OUT O_SQLSTATE	CHARACTER(5) )
  SPECIFIC SAVE_ALERT2
DYNAMIC RESULT SETS 0
  LANGUAGE SQL
-- #######################################################################################
--
-- FILE: SAVE_ALERT.SQL
--
--
-- DESCRIPTION:
--	THIS PROCEDURE WILL UPDATE THE ALERT STATUS AND CREATE AN ALERT ACTION RECORD
--
-- Called By:
--     WEB UI
--
-- Calls:
--     save_alert_fulfill
--
-- HISTORY:
--	SEPT 01, 2005 - INITIAL CREATION (MIKE AMES X10866 MICHAEL.AMES@SAS.COM)
--	SEPT 09, 2005 - ADDED BLOCK CODE ASSIGNOR & CHANGED NAME FROM SET ALERT
--                      TO SAVE ALERT(MIKE AMES X10866 MICHAEL.AMES@SAS.COM)
--	SEPT 15, 2005 - ADDED SERVICE_MOD, GROUP CODE, ACTION ACTION, BLOCK ACTION, PREV ACCOUNT BLOCK
--	DEC  09, 2005 - ADDED ROLLBACK LOGIC.
--	DEC  15, 2005 - FIXED ROLLBACK ERROR CODE.
--	JAN  16, 2006 - CHANGED FULFIL_TEXT_PROOF_CODE TO FULFIL_TEXT_PROOF_CODE_1
--                      ADDED FULFIL_TEXT_PROOF_CODE_2
--                      ADDED FULFIL_TEXT_PROOF_CODE_3
--                      LSTUPDT_NBR NOT HARD CODED TO 0 IN THE INSERT STATEMENT
--	FEB  27, 2006 - ADDED L_ACTION_TYPE = 'ANALYST ACTION' TO INSERT STATEMENT
--                      FOR FCM_ALERT_ACTION
--	MAR  30, 2006 - ADDED CALL TO PREP_FULFL_MSG PROC
--      2006/06/07    - ADDED 8 FULFILMENT PARAMETERS
--      2006/07/05    - ADDED CHECK FOR MEMO_TYPE != 'S'
--                        TO BYPASS THE PREP_FULFIL_MSG CALL
--      2006/10/31    - ADDED LOGIC TO CHANGE THE ACTION_TYPE AND USER_NAME
--                        ON AN AUTO SUBMISSION
--      2007/02/12    - ADDED STRATGEY_ID
--      2007/06/19    - ADAPTED FOR R2 (INITIAL PASS)
--      2007/07/17    - CHANGES FOR NEW FCM_ALERT_FULFILMENT TABLE
--      2007/09/17    - REMOVED CARD_ORG, CARD_LOGO, CARD_PORTFOLIO, CH_NAME_1, CH_1_MOBILE_1,
--                      CH_EMAIL_1,  CMS_LANG_CODE
--      2007/09/19    - LOGIC TO SPLIT BLOCK DATES AND TIMES ADDED
--      2008/02/07    - ADDED CODE TO SET VALUE FOR FINAL_DISPOSITION_TIMESTAMP AND STANDARDIZE
--                        CURRENT UTC TIMESTAMPS
--      2008/02/14    - ADDED CODE TO SET VALUE FOR FINAL_DISPOSITION_TIMESTAMP AND STANDARDIZE
--                        CURRENT UTC TIMESTAMPS--
--      2008/09/02    - ADDED ADDITIONAL WHERE CLAUSE CONDITION TO ENSURE THAT THE UPDATES ONLY
--                        UPDATE THE ALERT IF THE I_USER_NAME IS THE CHECKOUT_USER ALSO.
--      2009/08/14   LEUSHE   ADDED THE SECURITY CHECK TO ENSURE THE USER_NAME HAS ACCESS TO UPDATE THE ALERT
--      2009/09/10   LEUSHE Added new parameter to pass to save_alert_fulfil I_MERCH_CNTRY_NAMES 
--      2009/11/02   LEUSHE change the alert_action insert to use l_lstupdt_nbr instead of i_lstupdt_nbr
--      2010/03/30   LEUSHE Changed the call to save_alert_Fulfill to match the new proc for alert fulfullments
--                          Change the signature to not have all the alert fulfillment columns and added the xml msg input
--      2010/04/26   LEUSHE Changed to delete the alert_id from the FCM_ALERT_SUSPEND table
--	    2010/05/24   LEUSHE Added UNSUSPEND Alert Action insert
--      2010/10/22  leushe  Changed references to GRX_, GRA_ and GRC_ to XQO_,AQO_, and blank respectively
--      2010/10/29  leushe  added delete from the IVR_ALERT table if we don't rollback.
--      2011/02/28   leushe move the i_commit flag check to its own if block, so return parms will be set
--		2011/03/28   leushe made the action_type ANALYST ACTION when inserting UNSUSPEND Alert action
--      2011/03/30	 leushe commented out the insert into alert_action for unsuspend until futher reporting need makes it required
--      2011/04/01    leushe Changed the FCM_ALERT_SUSPEND to fix race condition and addition of ALERT_CHECKOUT_CNT column
-- NOTES:
--
--
-- USAGE:
--
--
-- TIPS:
--
-- #######################################################################################
SA: BEGIN

-- STEP 0A. DECLARE  PROCEDURE VARIABLES
DECLARE   P_ROW_COUNT INT DEFAULT -1;
DECLARE   P_ROW_COUNT_ACT INT DEFAULT -1;
DECLARE   P_ALERT_ACTION_ID BIGINT;
DECLARE   P_ALERT_ERROR INT DEFAULT 0;
DECLARE   P_ALERT_ACTION_ERROR INT DEFAULT 0;

-- DECLARE CONDITIONS
DECLARE SQLSTATE CHAR(5) DEFAULT '00000';
DECLARE SQLCODE  INT     DEFAULT 0;
DECLARE L_SQLMSG CHAR(200) DEFAULT NULL;



-- STEP 0B. DECLARE  AND INTIALIZE ALERT VARIABLES
DECLARE   L_ALERT_ID BIGINT;
DECLARE   L_AQO_NUM  CHARACTER(34) ;
DECLARE   L_ALERT_VALUE  CHARACTER(34) ;
DECLARE   L_XQO_NUM  CHARACTER(34) ;
DECLARE   L_ALRT_VER_NUM  INTEGER ;
DECLARE   L_QUEUE_ID BIGINT ;
DECLARE   L_MULTI_ORG_ID INTEGER;
DECLARE   L_ALERT_TYPE CHAR(1);
DECLARE   L_RESULT_ID  INTEGER;
DECLARE   L_ALERT_STATUS CHARACTER(10) ;
DECLARE   L_GMT_OFFSET DECIMAL(3,1) DEFAULT 0.0;
DECLARE   L_FRAUD_TYPE_CODE  CHARACTER(3);
DECLARE   L_PEND_REASON_CODE CHARACTER(3);
DECLARE   L_FRAUD_SCORE  INTEGER;
DECLARE   L_FRAUD_SCORE_REASON_CODE_1  INTEGER;
DECLARE   L_FRAUD_SCORE_REASON_CODE_2   INTEGER;
DECLARE   L_FRAUD_SCORE_REASON_CODE_3   INTEGER;
DECLARE   L_INIT_FRAUD_SCORE INTEGER;
DECLARE   L_INIT_FRAUD_SCORE_REASON_CODE_1  INTEGER;
DECLARE   L_INIT_FRAUD_SCORE_REASON_CODE_2  INTEGER;
DECLARE   L_INIT_FRAUD_SCORE_REASON_CODE_3  INTEGER;
DECLARE   L_HWM_FRAUD_SCORE  INTEGER;
DECLARE   L_HWM_FRAUD_SCORE_REASON_CODE_1   INTEGER;
DECLARE   L_HWM_FRAUD_SCORE_REASON_CODE_2   INTEGER;
DECLARE   L_HWM_FRAUD_SCORE_REASON_CODE_3   INTEGER;
DECLARE   L_ALERT_SERVICE_CNT  INTEGER  ;
DECLARE   L_ALERT_ORIGIN VARCHAR(20) ;
DECLARE   L_LST_TRANSACTION_AMT  DECIMAL(19,5);
DECLARE   L_LST_TRANSACTION_TIMESTAMP  TIMESTAMP;
DECLARE   L_USER_SERVICE_TIMESTAMP TIMESTAMP;
DECLARE   L_SYSTEM_SERVICE_TIMESTAMP TIMESTAMP;
DECLARE   L_NEXT_SERVICE_TIMESTAMP TIMESTAMP;
DECLARE   L_QUEUE_ENTRY_TIMESTAMP  TIMESTAMP;
DECLARE   L_FINAL_DISPOSITION_TIMESTAMP  TIMESTAMP;
DECLARE   L_CHECKOUT_USER  CHARACTER(60) ;
DECLARE   L_CHECKOUT_TIMESTAMP TIMESTAMP;
DECLARE   L_CREATE_USER  CHARACTER(60) ;
DECLARE   L_CREATE_TIMESTAMP TIMESTAMP;
DECLARE   L_LSTUPDT_USER CHARACTER(60);
DECLARE   L_LSTUPDT_TIMESTAMP  TIMESTAMP;
DECLARE   L_LSTUPDT_NBR  SMALLINT ;
DECLARE   L_LOGICAL_DEL_IND  CHARACTER(1);
DECLARE   L_ACTION_TYPE CHAR(20);
DECLARE   L_BASE_RULE_ID BIGINT ;
DECLARE   L_RULE_VERSION_NBR INTEGER ;
DECLARE   L_PAUSE_TIMESTAMP TIMESTAMP;
DECLARE   L_PAUSE_ELAPSED_TIME INTEGER;
DECLARE   L_ID BIGINT ;
DECLARE   L_ALERT_ACTION_ID BIGINT ;
DECLARE   L_USER_NAME CHARACTER(60);
DECLARE   L_SYS_BLOCK_CODE_1 CHAR(8);
DECLARE   L_TRANS_BLOCK_CODE CHAR(8);
DECLARE   L_RESURFACE_TIMESTAMP TIMESTAMP;
DECLARE   L_SENT_SECURE_MESSAGE_IND CHAR(1);
DECLARE   L_INVALID_CONTACT_IND CHAR(1);
DECLARE   L_SYS_BLOCK_CODE_1_DATE DATE;
DECLARE   L_SYS_BLOCK_CODE_1_TIME TIME;
DECLARE   L_SYS_BLOCK_1_UNTIL_DATE DATE;
DECLARE   L_SYS_BLOCK_1_UNTIL_TIME TIME;
DECLARE   L_TRANS_BLOCK_DATE DATE ;
DECLARE   L_TRANS_BLOCK_TIME TIME;
DECLARE   L_TRANS_BLOCK_UNTIL_DATE DATE;
DECLARE   L_TRANS_BLOCK_UNTIL_TIME TIME;

DECLARE L_CNTR SMALLINT;
DECLARE L_CURRENT_UTC_TIMESTAMP TIMESTAMP;
SET L_CURRENT_UTC_TIMESTAMP = CURRENT TIMESTAMP - CURRENT TIMEZONE;
     
     -- STEP 0C. DECLARE  AND INITALIZE ACCOUNT VARIABLES
     -- STEP 1.  UPDATE ALERT
SAVEPOINT STRT_SAVEALERT ON ROLLBACK RETAIN CURSORS;

IF ( CHECK_ALERT_ACCESS(I_USER_NAME,I_ALERT_ID) = 1 ) THEN     
 
     SET L_ACTION_TYPE = 'ANALYST ACTION';
     SET L_USER_NAME = I_USER_NAME;
     
     
     -- 2007/09/19
     IF LENGTH(RTRIM(I_SYS_BLOCK_CODE_1)) = 0 THEN
      SET L_SYS_BLOCK_CODE_1_DATE = NULL ;
      SET L_SYS_BLOCK_CODE_1_TIME = NULL ;
     ELSE
      SET L_SYS_BLOCK_CODE_1_DATE = DATE(L_CURRENT_UTC_TIMESTAMP ) ;
      SET L_SYS_BLOCK_CODE_1_TIME = TIME(L_CURRENT_UTC_TIMESTAMP ) ;
     END IF;
     
     SET L_SYS_BLOCK_1_UNTIL_DATE = DATE(I_SYS_BLOCK_CODE_1_TS  );
     SET L_SYS_BLOCK_1_UNTIL_TIME = TIME(I_SYS_BLOCK_CODE_1_TS  ) ;
     
     IF LENGTH(RTRIM(I_TRANS_BLOCK_CODE)) = 0 THEN
      SET L_TRANS_BLOCK_DATE = NULL ;
      SET L_TRANS_BLOCK_TIME = NULL ;
     ELSE
      SET L_TRANS_BLOCK_DATE = DATE(L_CURRENT_UTC_TIMESTAMP ) ;
      SET L_TRANS_BLOCK_TIME = TIME(L_CURRENT_UTC_TIMESTAMP ) ;
     END IF;
     
     SET L_TRANS_BLOCK_UNTIL_DATE = DATE(I_TRANS_BLOCK_TS  );
     SET L_TRANS_BLOCK_UNTIL_TIME = TIME(I_TRANS_BLOCK_TS  );
     -- 2008/02/07:
     --   SET FINAL_DISPOSITION_TIMESTAMP TO CURRENT UTC TIMESTAMP ONLY IF
     --    ALERT STATUSED VERIFIED ACTIVITY (6) OR CONFIRMED FRAUD.(7).
     --    VALUE IS SET TO NULL OTHERWISE TO OVERWRITE PREVIOUSLY SET VALUE.
     --IF (I_RESULT_ID = 6) OR (I_RESULT_ID = 7) THEN
     
     IF I_RESULT_ID IN (6,7) THEN
      SET L_FINAL_DISPOSITION_TIMESTAMP = L_CURRENT_UTC_TIMESTAMP;
     ELSE
      SET L_FINAL_DISPOSITION_TIMESTAMP = NULL;
     END IF;
     
     
     -- STEP 1A. IF RE-ROUTE UPDATE QUEUE ID, AND QUEUE ENTRY TIMESTAMP. & CHECKIN ALERT
     IF I_RESULT_ID = 5 THEN
         UPDATE FCM_ALERT
         SET
            QUEUE_ID=I_REROUTE_QUEUE_ID,
            RESULT_ID=I_RESULT_ID,
            ALERT_STATUS='ACTIVE',
            FRAUD_TYPE_CODE=I_FRAUD_TYPE_CODE,
--            PEND_REASON_CODE=I_PEND_REASON_CODE,
            NEXT_SERVICE_TIMESTAMP=I_NEXT_SERVICE_TIMESTAMP,
            QUEUE_ENTRY_TIMESTAMP=L_CURRENT_UTC_TIMESTAMP ,
            FINAL_DISPOSITION_TIMESTAMP = L_FINAL_DISPOSITION_TIMESTAMP,
            CHECKOUT_USER=NULL,
            CHECKOUT_TIMESTAMP=NULL,
            LSTUPDT_USER=L_USER_NAME,
            LSTUPDT_TIMESTAMP=L_CURRENT_UTC_TIMESTAMP ,
            LSTUPDT_NBR=LSTUPDT_NBR+1
         WHERE
            ALERT_ID = I_ALERT_ID
            AND LSTUPDT_NBR = I_LSTUPDT_NBR
            AND CHECKOUT_USER = L_USER_NAME;
         
         GET DIAGNOSTICS P_ROW_COUNT = ROW_COUNT;
         -- IF P_ROW_COUNT NE 1 THEN RAISE EXCEPTION
         IF P_ROW_COUNT = 0 THEN
          -- SIGNAL C_ZERO_ROWS_UPDATED
          -- SET MESSAGE_TEXT = 'ZERO ALERTS UPDATED:';
          -- SET O_SQLSTATE = 'ERROR';
          -- SET O_SQLCODE  = 1;
            SET P_ALERT_ERROR = 1;
         END IF;
     --END IF;
     ELSE
         -- STEP 1B. ALL OTHER RESULTS
         --IF I_RESULT_ID != 5 THEN
         UPDATE FCM_ALERT
         SET
            RESULT_ID=I_RESULT_ID,
            ALERT_STATUS='ACTIVE',
            FRAUD_TYPE_CODE=I_FRAUD_TYPE_CODE,
--            PEND_REASON_CODE=I_PEND_REASON_CODE,
            NEXT_SERVICE_TIMESTAMP=I_NEXT_SERVICE_TIMESTAMP,
            FINAL_DISPOSITION_TIMESTAMP = L_FINAL_DISPOSITION_TIMESTAMP,
            CHECKOUT_USER=NULL,
            CHECKOUT_TIMESTAMP=NULL,
            LSTUPDT_USER=L_USER_NAME,
            LSTUPDT_TIMESTAMP=L_CURRENT_UTC_TIMESTAMP ,
            LSTUPDT_NBR=LSTUPDT_NBR+1
         WHERE
            ALERT_ID = I_ALERT_ID
            AND LSTUPDT_NBR = I_LSTUPDT_NBR
            AND CHECKOUT_USER = L_USER_NAME;
         
         GET DIAGNOSTICS P_ROW_COUNT = ROW_COUNT;
         
         IF P_ROW_COUNT < 1 THEN
          -- SIGNAL C_ZERO_ROWS_UPDATED
          -- SET MESSAGE_TEXT = 'ZERO ALERTS UPDATED:';
          -- SET O_SQLSTATE = 'ERROR';
          -- SET O_SQLCODE  = 1;
            SET P_ALERT_ERROR = 1;
         END IF;
     END IF;
     
     
     -- STEP 2. INSERT ALERT ACTION
     -- STEP 2A. GET ALERT
     IF  P_ALERT_ERROR = 0 THEN
     
     
          SELECT
              ALERT_ID,
              ALERT_VERSION_NBR, --NEW V3
              AQO_ACCT_NUM,
              ALERT_VALUE,
              XQO_CUST_NUM, --NEW V3
              QUEUE_ID,
              MULTI_ORG_ID,
              ALERT_TYPE,
              RESULT_ID,
              ALERT_STATUS,
              GMT_OFFSET,
              FRAUD_TYPE_CODE,
              PEND_REASON_CODE,
              FRAUD_SCORE,
              FRAUD_SCORE_REASON_CODE_1,
              FRAUD_SCORE_REASON_CODE_2,
              FRAUD_SCORE_REASON_CODE_3,
              INIT_FRAUD_SCORE,
              INIT_FRAUD_SCORE_REASON_CODE_1,
              INIT_FRAUD_SCORE_REASON_CODE_2,
              INIT_FRAUD_SCORE_REASON_CODE_3,
              HWM_FRAUD_SCORE,
              HWM_FRAUD_SCORE_REASON_CODE_1,
              HWM_FRAUD_SCORE_REASON_CODE_2,
              HWM_FRAUD_SCORE_REASON_CODE_3,
              ALERT_SERVICE_CNT,
              ALERT_ORIGIN,
              LST_TRANSACTION_AMT,
              LST_TRANSACTION_TIMESTAMP,
              USER_SERVICE_TIMESTAMP,
              SYSTEM_SERVICE_TIMESTAMP,
              NEXT_SERVICE_TIMESTAMP,
              QUEUE_ENTRY_TIMESTAMP,
              FINAL_DISPOSITION_TIMESTAMP,
              CHECKOUT_USER,
              CHECKOUT_TIMESTAMP,
              CREATE_USER,
              CREATE_TIMESTAMP,
              LSTUPDT_USER,
              LSTUPDT_TIMESTAMP,
              LSTUPDT_NBR,
              LOGICAL_DEL_IND,
              BASE_RULE_ID,
              RULE_VERSION_NBR,
              PAUSE_TIMESTAMP,
              PAUSE_ELAPSED_TIME
          INTO
              L_ALERT_ID,
              L_ALRT_VER_NUM,
              L_AQO_NUM ,
              L_ALERT_VALUE ,
              L_XQO_NUM ,
              L_QUEUE_ID,
              L_MULTI_ORG_ID,
              L_ALERT_TYPE,
              L_RESULT_ID ,
              L_ALERT_STATUS,
              L_GMT_OFFSET,
              L_FRAUD_TYPE_CODE ,
              L_PEND_REASON_CODE,
              L_FRAUD_SCORE ,
              L_FRAUD_SCORE_REASON_CODE_1 ,
              L_FRAUD_SCORE_REASON_CODE_2 ,
              L_FRAUD_SCORE_REASON_CODE_3 ,
              L_INIT_FRAUD_SCORE,
              L_INIT_FRAUD_SCORE_REASON_CODE_1,
              L_INIT_FRAUD_SCORE_REASON_CODE_2,
              L_INIT_FRAUD_SCORE_REASON_CODE_3,
              L_HWM_FRAUD_SCORE ,
              L_HWM_FRAUD_SCORE_REASON_CODE_1 ,
              L_HWM_FRAUD_SCORE_REASON_CODE_2 ,
              L_HWM_FRAUD_SCORE_REASON_CODE_3 ,
              L_ALERT_SERVICE_CNT ,
              L_ALERT_ORIGIN,
              L_LST_TRANSACTION_AMT ,
              L_LST_TRANSACTION_TIMESTAMP ,
              L_USER_SERVICE_TIMESTAMP,
              L_SYSTEM_SERVICE_TIMESTAMP,
              L_NEXT_SERVICE_TIMESTAMP,
              L_QUEUE_ENTRY_TIMESTAMP ,
              L_FINAL_DISPOSITION_TIMESTAMP ,
              L_CHECKOUT_USER ,
              L_CHECKOUT_TIMESTAMP,
              L_CREATE_USER ,
              L_CREATE_TIMESTAMP,
              L_LSTUPDT_USER,
              L_LSTUPDT_TIMESTAMP ,
              L_LSTUPDT_NBR,
              L_LOGICAL_DEL_IND,
              L_BASE_RULE_ID,
              L_RULE_VERSION_NBR,
              L_PAUSE_TIMESTAMP,
              L_PAUSE_ELAPSED_TIME
          FROM FCM_ALERT
          WHERE
            ALERT_ID = I_ALERT_ID;
          
          
          -- STEP 3B. INSERT ACTION
          INSERT INTO FCM_ALERT_ACTION (
              ALERT_ID,
              ALERT_VERSION_NBR,
              ACTION_TIMESTAMP,
              AQO_ACCT_NUM,
              ALERT_VALUE,
              XQO_CUST_NUM,
              QUEUE_ID,
              MULTI_ORG_ID,
              ALERT_TYPE,
              RESULT_ID,
              ALERT_STATUS,
              GMT_OFFSET,
              FRAUD_TYPE_CODE,
              FRAUD_SCORE,
              FRAUD_SCORE_REASON_CODE_1,
              FRAUD_SCORE_REASON_CODE_2,
              FRAUD_SCORE_REASON_CODE_3,
              INIT_FRAUD_SCORE,
              INIT_FRAUD_SCORE_REASON_CODE_1,
              INIT_FRAUD_SCORE_REASON_CODE_2,
              INIT_FRAUD_SCORE_REASON_CODE_3,
              HWM_FRAUD_SCORE,
              HWM_FRAUD_SCORE_REASON_CODE_1,
              HWM_FRAUD_SCORE_REASON_CODE_2,
              HWM_FRAUD_SCORE_REASON_CODE_3,
              ALERT_SERVICE_CNT,
              ALERT_ORIGIN,
              LST_TRANSACTION_AMT,
              LST_TRANSACTION_TIMESTAMP,
              USER_SERVICE_TIMESTAMP,
              SYSTEM_SERVICE_TIMESTAMP,
              NEXT_SERVICE_TIMESTAMP,
              QUEUE_ENTRY_TIMESTAMP,
              FINAL_DISPOSITION_TIMESTAMP,
              CHECKOUT_USER,
              CHECKOUT_TIMESTAMP,
              CREATE_USER,
              CREATE_TIMESTAMP,
              LSTUPDT_USER,
              LSTUPDT_TIMESTAMP,
              LSTUPDT_NBR,
              SYS_BLOCK_CODE_1_DATE, -- CHANGE
              SYS_BLOCK_CODE_1_TIME, -- ADDED 2/14
              SYS_BLOCK_1_UNTIL_DATE, -- ADDED 2/14
              SYS_BLOCK_1_UNTIL_TIME, -- ADDED 2/14
              SYS_BLOCK_CODE_1, -- CHANGE
              -- CHANGE   GROUP_CODE,
              GROUP_CODE,
              TRANS_BLOCK_DATE, -- CHANGE
              TRANS_BLOCK_TIME, -- ADDED 2/14
              TRANS_BLOCK_UNTIL_DATE, -- ADDED 2/14
              TRANS_BLOCK_UNTIL_TIME, -- ADDED 2/14
              TRANS_BLOCK_CODE, -- CHANGE
              CONTACT_HOME_PHONE_IND,
              CONTACT_MOBILE_PHONE_IND,
              CONTACT_WORK_PHONE_IND,
              REROUTE_QUEUE_ID,
              RESURFACE_TIMESTAMP,
              SERVICE_METHOD_CODE,
              SYS_BLOCK_1_UNTIL_ACTION, -- CHANGE
              PREVIOUS_SYS_BLOCK_CODE_1, -- CHANGE
              TRANS_BLOCK_UNTIL_ACTION, -- CHANGE
              ACTION_TYPE,
              BASE_RULE_ID,
              RULE_VERSION_NBR,
              PAUSE_TIMESTAMP,
              PAUSE_ELAPSED_TIME,
              OPEN_TO_BUY_AMT,
              STRATEGY_ID

           )
          VALUES(
              L_ALERT_ID,
              L_ALRT_VER_NUM,
              L_CURRENT_UTC_TIMESTAMP ,
              L_AQO_NUM,
              L_ALERT_VALUE,
              L_XQO_NUM ,
              L_QUEUE_ID,
              L_MULTI_ORG_ID,
              L_ALERT_TYPE,
              L_RESULT_ID ,
              L_ALERT_STATUS,
              L_GMT_OFFSET,
              L_FRAUD_TYPE_CODE ,
              L_FRAUD_SCORE ,
              L_FRAUD_SCORE_REASON_CODE_1 ,
              L_FRAUD_SCORE_REASON_CODE_2 ,
              L_FRAUD_SCORE_REASON_CODE_3 ,
              L_INIT_FRAUD_SCORE,
              L_INIT_FRAUD_SCORE_REASON_CODE_1,
              L_INIT_FRAUD_SCORE_REASON_CODE_2,
              L_INIT_FRAUD_SCORE_REASON_CODE_3,
              L_HWM_FRAUD_SCORE ,
              L_HWM_FRAUD_SCORE_REASON_CODE_1 ,
              L_HWM_FRAUD_SCORE_REASON_CODE_2 ,
              L_HWM_FRAUD_SCORE_REASON_CODE_3 ,
              L_ALERT_SERVICE_CNT ,
              L_ALERT_ORIGIN,
              L_LST_TRANSACTION_AMT ,
              L_LST_TRANSACTION_TIMESTAMP ,
              L_USER_SERVICE_TIMESTAMP,
              L_SYSTEM_SERVICE_TIMESTAMP,
              L_NEXT_SERVICE_TIMESTAMP,
              L_QUEUE_ENTRY_TIMESTAMP ,
              L_FINAL_DISPOSITION_TIMESTAMP ,
              L_CHECKOUT_USER ,
              L_CHECKOUT_TIMESTAMP,
              L_CREATE_USER ,
              L_CREATE_TIMESTAMP,
              L_LSTUPDT_USER,
              L_LSTUPDT_TIMESTAMP,
              L_LSTUPDT_NBR,
              L_SYS_BLOCK_CODE_1_DATE, --!!
              L_SYS_BLOCK_CODE_1_TIME, -- ADDED 2/14
              L_SYS_BLOCK_1_UNTIL_DATE, -- ADDED 2/14
              L_SYS_BLOCK_1_UNTIL_TIME, -- ADDED 2/14
              I_SYS_BLOCK_CODE_1, --!!
              I_GROUP_CODE,
              L_TRANS_BLOCK_DATE, --CHANGED 2/14
              L_TRANS_BLOCK_TIME, -- ADDED 2/14
              L_TRANS_BLOCK_UNTIL_DATE, -- ADDED 2/14
              L_TRANS_BLOCK_UNTIL_TIME, -- ADDED 2/14
              I_TRANS_BLOCK_CODE, -- !!
              I_CONTACT_HOME_PHONE_IND,
              I_CONTACT_MOBILE_PHONE_IND,
              I_CONTACT_WORK_PHONE_IND,
              I_REROUTE_QUEUE_ID,
              I_RESURFACE_TIMESTAMP,
              I_SERVICE_METHOD_CODE,
              I_SYS_BLOCK_1_UNTIL_ACTION, --!!
              I_PREVIOUS_SYS_BLOCK_CODE_1, --!!
              I_TRANS_BLOCK_UNTIL_ACTION, --!!
              L_ACTION_TYPE,
              L_BASE_RULE_ID,
              L_RULE_VERSION_NBR,
              L_PAUSE_TIMESTAMP,
              L_PAUSE_ELAPSED_TIME,
              I_OPEN_TO_BUY_AMT,
              I_STRATEGY_ID

           );
          
          
           GET DIAGNOSTICS P_ROW_COUNT_ACT = ROW_COUNT;
           IF P_ROW_COUNT_ACT <> 1 THEN
            SET  P_ALERT_ACTION_ERROR = 1;
           END IF;
          
     
           -- 2007/07/16 ALERT FULFIL MODS
           SET L_ALERT_ACTION_ID = IDENTITY_VAL_LOCAL();
     
     
           CALL SAVE_ALERT_FULFIL(
               L_ALERT_ACTION_ID
               ,I_SENT_LETTER_IND
               ,I_SENT_EMAIL_IND
               ,I_SENT_SMS_IND
               ,I_MEMO_TEXT
               ,L_USER_NAME
               ,I_SENT_SECURE_MESSAGE_IND
               ,I_INVALID_CONTACT_DETAILS_IND
               ,I_FULFILL_SENT_CODE
               ,I_FULFILL_XML_MSG
               ,O_SQLCODE
               ,O_SQLSTATE
            );

/* commented out for race condition*/            
            Select ALERT_CHECKOUT_CNT into L_CNTR from fcm_alert_suspend where xqo_cust_num = L_XQO_NUM and suspend_user = L_USER_NAME;
			If (L_CNTR = 1) then
				delete from fcm_alert_suspend where xqo_cust_Num = L_XQO_NUM and suspend_user = L_USER_NAME;
			else if (L_CNTR > 1) THEN
				update fcm_alert_suspend
			  		set ALERT_CHECKOUT_CNT = ALERT_CHECKOUT_CNT -1 
			  		where xqo_cust_Num = L_XQO_NUM and suspend_user = L_USER_NAME;
			  end if;
			end if;
            --Delete from FCM_ALERT_SUSPEND
           -- DELETE FROM FCM_ALERT_SUSPEND where ALERT_ID = I_ALERT_ID;
            -- Unsuspend the alert
--           INSERT INTO FCM_ALERT_ACTION (ALERT_ID ,
--			ALERT_VERSION_NBR ,
--			ACTION_TIMESTAMP ,
--			XQO_CUST_NUM ,
--			ALERT_VALUE ,
--			ALERT_TYPE ,
--			ACTION_TYPE,
--			MULTI_ORG_ID, 
--			ALERT_STATUS ,
--			GMT_OFFSET ,
--			ALERT_SERVICE_CNT ,
--			ALERT_ORIGIN ,
--			CREATE_USER ,
--			CREATE_TIMESTAMP ) 
--			SELECT a.ALERT_ID ,
--				a.ALERT_VERSION_NBR ,
--				CURRENT TIMESTAMP - CURRENT TIMEZONE ,
--				a.XQO_CUST_NUM ,
--				a.ALERT_VALUE ,
--				a.ALERT_TYPE ,
--				'ANALYST ACTION',
--				a.MULTI_ORG_ID, 
--				'UNSUSPEND' ,
--				a.GMT_OFFSET ,
--				a.ALERT_SERVICE_CNT ,
--				a.ALERT_ORIGIN ,
--				a.CREATE_USER ,
--				a.CREATE_TIMESTAMP  
--			FROM FCM_ALERT a
--			WHERE a.ALERT_ID = I_ALERT_ID
--			And not exists(select 1 from fcm_alert_suspend where xqo_cust_num = a.xqo_cust_num)
--

           

          
     END IF;
END IF;

IF ( P_ROW_COUNT <> 1 OR P_ROW_COUNT_ACT <> 1  ) THEN
 ROLLBACK TO SAVEPOINT STRT_SAVEALERT;
 SET O_SQLSTATE = 'ERROR';
 SET O_SQLCODE  = 1;
    SET O_ALERT_ACTION_ID = -1;
END IF;

IF (P_ROW_COUNT = 1 AND P_ROW_COUNT_ACT = 1) THEN
 delete from fcm_ivr_alert where alert_id = i_alert_id;
 SET O_SQLSTATE = 'SUCESS';
 SET O_SQLCODE  = 0;
 SET O_ALERT_ACTION_ID = L_ALERT_ACTION_ID ;
END IF;

IF (I_NEED_TO_COMMIT = 'Y') THEN
 COMMIT;
END IF;

END SA@

--GRANT EXECUTE ON SPECFIC PROCEDURE SAVE_ALERT2 TO ROLE APPROLE@

